/*
 * File:   Optical Flow with Hetmite
 * Author: Fabiola Pacheco
 * 
 * Created on 04/18/2017
 * Last modification on 10/11/2017
 */
#ifndef itkHermiteOpticalFlow_h
#define itkHermiteOpticalFlow_h
 
#include "itkImageToImageFilter.h"
#include <itkArray.h>
#include "itkVariableLengthVector.h"
#include <vnl/vnl_vector.h>

namespace itk
{
  template< class TInputImage, class TOutputImage>
  class HermiteOpticalFlow : public ImageToImageFilter< TInputImage, TOutputImage >
  {
  public:
    /** Standard class typedefs. */
    typedef HermiteOpticalFlow Self;
    typedef ImageToImageFilter< TInputImage, TOutputImage > Superclass;
    typedef SmartPointer< Self > Pointer;
   
    /** Method for creation through the object factory. */
    itkNewMacro(Self);
   
    /** Run-time type information (and related methods). */
    itkTypeMacro(HermiteOpticalFlow, ImageToImageFilter);
   
    /** Set first Image*/
    void SetInputImage0(const TInputImage* image0);
   
    /** Set second Image, same size as the first Image */
    void SetInputImage1(const TInputImage* image1);

    /** Set number of nivels for scale images, defaul is 1 
    *\with scale 0, the originals images are not scaled
    *\with scale > 0, the originals images are not considered for multiresolution calculation
    */
    void SetScale(const int scale);

    /** Set number of iterations, defaul is 1 */
    void SetIterations(const int iter);

    /**Outputs*/
    TOutputImage* GetDu();
    TOutputImage* GetDv();
   
  protected:
    HermiteOpticalFlow();
    ~HermiteOpticalFlow(){}
   
    /** Does the real work. */
    virtual void GenerateData();

    /** Create the Output */
    DataObject::Pointer MakeOutput();
   
  private:
    HermiteOpticalFlow(const Self &); //purposely not implemented
    void operator=(const Self &);  //purposely not implemented

    /** Calcule optical flow betwen 2 images 
    *\OpticalFlowHermite(Img0,Img1,du,dv)
    */  
    void OpticalFlowHermite(const InputImageType*,
		                    const InputImageType*,
							itk::Image<double, 2>::Pointer,
							itk::Image<double, 2>::Pointer);
    
    /** Calcule gradient of image */
    void Gradient(itk::Image<double, 2>::Pointer,bool);
    //if flag == false
    itk::Image<double, 2>::Pointer GradientX0; 
    itk::Image<double, 2>::Pointer GradientY0; 
    //if flag == true
    itk::Image<double, 2>::Pointer GradientX1; 
    itk::Image<double, 2>::Pointer GradientY1;

    /** Solve duv for system: A duv = b
    *\SOR(rows of matrix A,colums of matrix A,values of matrix A, vector b, vector duv)    
    *\A is a sparse matrix
    */
    void SOR(itk::Array<unsigned int>,
		     itk::Array<int>,itk::Array<double>, 
			 vnl_vector<double>,vnl_vector<double>);

    itk::Image<double, 2>::Pointer sorDu;
    itk::Image<double, 2>::Pointer sorDv;

    TOutputImage* GetNthOutput(int n);

    int imgScale;

    int itera_outer;   
  };

} //namespace ITK
  
#ifndef ITK_MANUAL_INSTANTIATION
#include "itkHermiteOpticalFlow.txx"
#endif
  
#endif // itkHermiteOpticalFlow_h