/*
 * File:   Optical Flow with Hetmite
 * Author: Fabiola Pacheco
 * 
 * Created on 04/18/2017
 * Last modification on 10/11/2017
 */
#ifndef itkHermiteOpticalFlow_txx
#define itkHermiteOpticalFlow_txx
 
#include "itkHermiteOpticalFlow.h"
 
#include "itkObjectFactory.h"
#include "itkHermiteImageFilter.h"
#include "itkIdentityTransform.h"
#include "itkLinearInterpolateImageFunction.h"
#include "itkResampleImageFilter.h"
#include "itkImageRegionIterator.h"
#include "itkImageRegionConstIterator.h"
#include "itkImageRegionIteratorWithIndex.h"
#include "itkContinuousIndex.h"
#include <itkVariableSizeMatrix.h>
#include <itkMatrix.h>

#include "vnl/vnl_math.h"
#include "vnl/vnl_matrix.h"

#include <vnl/vnl_sparse_matrix.h>
#include <vnl/algo/vnl_sparse_lu.h>

 
namespace itk
{
 
  template< class TInputImage, class TOutputImage>
  HermiteOpticalFlow<TInputImage, TOutputImage>::HermiteOpticalFlow()
  {
    this->SetNumberOfRequiredInputs(2);
    this->SetNumberOfRequiredOutputs(2);
    
    this->SetNthOutput( 0, this->MakeOutput() );
    this->SetNthOutput( 1, this->MakeOutput() );
    
    imgScale = 1;
    
    itera_outer = 1;
  }
   
  template< class TInputImage, class TOutputImage>
  void HermiteOpticalFlow<TInputImage, TOutputImage>::
    SetInputImage0(const TInputImage* image0)
  {
    this->SetNthInput(0, const_cast<TInputImage*>(image0));
  }
   
  template< class TInputImage, class TOutputImage>
  void HermiteOpticalFlow<TInputImage, TOutputImage>::
    SetInputImage1(const TInputImage* image1)
  {
    this->SetNthInput(1, const_cast<TInputImage*>(image1));
  }
   
  template< class TInputImage, class TOutputImage>
  void HermiteOpticalFlow<TInputImage, TOutputImage>::
    SetScale(const int scale)
  {
    imgScale = scale;
  }
  
  template< class TInputImage, class TOutputImage>
  void HermiteOpticalFlow<TInputImage, TOutputImage>::
    SetIterations(const int iter)
  {
    itera_outer = iter;
  }
  
  template< class TInputImage, class TOutputImage>
  void HermiteOpticalFlow<TInputImage, TOutputImage>::GenerateData()
  {
    for(int i=0;i<2;i++){
      OutputImageType::Pointer temp = this->GetNthOutput(i);//Du and Dv
      temp->SetRegions(this->GetInput(0)->GetLargestPossibleRegion());
      temp->Allocate(); 
      temp->FillBuffer(0.0); // initialize it to all dark
    }
    
    itk::Image<double, 2>::Pointer tempDu = itk::Image<double, 2>::New();
    tempDu = itk::Image<double, 2>::New();
    tempDu->SetRegions(this->GetInput(0)->GetLargestPossibleRegion());
    tempDu->Allocate();
    tempDu->FillBuffer(0.0);
    
    itk::Image<double, 2>::Pointer tempDv = itk::Image<double, 2>::New();
    tempDv = itk::Image<double, 2>::New();
    tempDv->SetRegions(this->GetInput(0)->GetLargestPossibleRegion());
    tempDv->Allocate();
    tempDv->FillBuffer(0.0);
    
    if(imgScale==0)
    {
      OpticalFlowHermite(this->GetInput(0),this->GetInput(1),tempDu,tempDv);
      
      //outputs Du and Dv
      itk::ImageRegionIterator< itk::Image<double, 2> > sorDuIterator
        (sorDu,sorDu->GetLargestPossibleRegion());
      itk::ImageRegionIterator< itk::Image<double, 2> > sorDvIterator
        (sorDv,sorDv->GetLargestPossibleRegion());
      itk::ImageRegionIterator< TOutputImage > outputIteratorU
        (this->GetNthOutput(0),this->GetNthOutput(0)->GetLargestPossibleRegion());
      itk::ImageRegionIterator< TOutputImage > outputIteratorV
        (this->GetNthOutput(1),this->GetNthOutput(1)->GetLargestPossibleRegion());
      
      while(!outputIteratorU.IsAtEnd()){
        outputIteratorU.Set(sorDuIterator.Get());
        outputIteratorV.Set(sorDvIterator.Get());
        
        ++outputIteratorU;
        ++outputIteratorV;
        ++sorDuIterator;
        ++sorDvIterator;
      }
      
    }else{      
        
      //Sampling      
      InputImageType::SizeType outputSize;
      InputImageType::SpacingType outputSpacing;
      
      // linearly interpolates image intensity at a non-integer pixel position
      itk::LinearInterpolateImageFunction<InputImageType, double >::Pointer interpolatorIn 
        = itk::LinearInterpolateImageFunction<InputImageType, double >::New();  
      itk::LinearInterpolateImageFunction<OutputImageType, double >::Pointer interpolatorOut 
        = itk::LinearInterpolateImageFunction<OutputImageType, double >::New();  
      itk::LinearInterpolateImageFunction<InputImageType, double >::Pointer interpolatorIm1 
        = itk::LinearInterpolateImageFunction<InputImageType, double >::New();  
    
      itk::ResampleImageFilter<InputImageType,InputImageType>::Pointer resampleImg0 
        = itk::ResampleImageFilter<InputImageType,InputImageType>::New();
      resampleImg0->SetTransform(itk::IdentityTransform<double, 2>::New());
      resampleImg0->SetInterpolator(interpolatorIn);
      
      itk::ResampleImageFilter<InputImageType,InputImageType>::Pointer resampleImg1 
        = itk::ResampleImageFilter<InputImageType,InputImageType>::New();
      resampleImg1->SetTransform(itk::IdentityTransform<double, 2>::New());
      resampleImg1->SetInterpolator(interpolatorIn);
      
      itk::ResampleImageFilter<OutputImageType,OutputImageType>::Pointer resampleDu 
        = itk::ResampleImageFilter<OutputImageType,OutputImageType>::New();
      resampleDu->SetTransform(itk::IdentityTransform<double, 2>::New());
      resampleDu->SetInterpolator(interpolatorOut);  
      
      itk::ResampleImageFilter<OutputImageType,OutputImageType>::Pointer resampleDv 
        = itk::ResampleImageFilter<OutputImageType,OutputImageType>::New();
      resampleDv->SetTransform(itk::IdentityTransform<double, 2>::New());
      resampleDv->SetInterpolator(interpolatorOut);  
        
///////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////scale factor////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////
      float factor = 0.8;
      
      outputSize[0] = this->GetInput(0)->GetLargestPossibleRegion().GetSize()[0] 
        * pow(factor, imgScale);
      outputSize[1] = this->GetInput(0)->GetLargestPossibleRegion().GetSize()[1] 
        * pow(factor, imgScale);  
      
      outputSpacing[0] = this->GetInput(0)->GetSpacing()[0] 
        * (static_cast<double>(this->GetInput(0)->GetLargestPossibleRegion().GetSize()[0]) 
          / static_cast<double>(outputSize[0]));
      outputSpacing[1] = this->GetInput(0)->GetSpacing()[1] 
        * (static_cast<double>(this->GetInput(0)->GetLargestPossibleRegion().GetSize()[1]) 
          / static_cast<double>(outputSize[1]));  
        
      resampleDu->SetInput(tempDu);
      resampleDu->SetSize(outputSize);
      resampleDu->SetOutputSpacing(outputSpacing);
      resampleDu->UpdateLargestPossibleRegion();
      
      resampleDv->SetInput(tempDv);
      resampleDv->SetSize(outputSize);
      resampleDv->SetOutputSpacing(outputSpacing);
      resampleDv->UpdateLargestPossibleRegion();
              
      resampleImg0->SetInput(this->GetInput(0));
      resampleImg0->SetSize(outputSize);
      resampleImg0->SetOutputSpacing(outputSpacing);
      resampleImg0->UpdateLargestPossibleRegion();
      
      resampleImg1->SetInput(this->GetInput(1));
      resampleImg1->SetSize(outputSize);
      resampleImg1->SetOutputSpacing(outputSpacing);
      resampleImg1->UpdateLargestPossibleRegion();
      
      //interpolated image1
      InputImageType::Pointer interpImg1 = InputImageType::New();
      interpImg1 = resampleImg1->GetOutput();
      
      itk::Image<double, 2>::Pointer auxDu = itk::Image<double, 2>::New();
      itk::Image<double, 2>::Pointer auxDv = itk::Image<double, 2>::New();
      auxDu=resampleDu->GetOutput();
      auxDv=resampleDv->GetOutput();
      
      itk::ContinuousIndex<double, 2> pixel;
      
      //optical folw multyscale, big to small
      for(int s=imgScale;s>0;s--){
        
        //calcule optical flow
        OpticalFlowHermite(resampleImg0->GetOutput(),interpImg1,auxDu,auxDv);
          
        //sum outputs 
        itk::ImageRegionIterator< itk::Image<double, 2> > sorDuIterator
          (sorDu,sorDu->GetLargestPossibleRegion());
        itk::ImageRegionIterator< itk::Image<double, 2> > sorDvIterator
          (sorDv,sorDv->GetLargestPossibleRegion());
        itk::ImageRegionIterator< itk::Image<double, 2> > auxDuIterator
          (auxDu,auxDu->GetLargestPossibleRegion());
        itk::ImageRegionIterator< itk::Image<double, 2> > auxDvIterator
          (auxDv,auxDv->GetLargestPossibleRegion());
        
        while(!auxDuIterator.IsAtEnd()){

          auxDuIterator.Set(auxDuIterator.Get()+sorDuIterator.Get());
          auxDvIterator.Set(auxDvIterator.Get()+sorDvIterator.Get());
          
          ++sorDuIterator;
          ++sorDvIterator;
          ++auxDuIterator;
          ++auxDvIterator;

        }
        
        //upsample outputs        
        outputSize[0] = this->GetInput(0)->GetLargestPossibleRegion().GetSize()[0] 
          * pow(factor, s-1);
        outputSize[1] = this->GetInput(0)->GetLargestPossibleRegion().GetSize()[1] 
          * pow(factor, s-1);  
          
        outputSpacing[0] = this->GetInput(0)->GetSpacing()[0] * (static_cast<double>
          (this->GetInput(0)->GetLargestPossibleRegion().GetSize()[0])/ 
          static_cast<double>(outputSize[0]));
        outputSpacing[1] = this->GetInput(0)->GetSpacing()[1] * 
          (static_cast<double>(this->GetInput(0)->
          GetLargestPossibleRegion().GetSize()[1])/ 
          static_cast<double>(outputSize[1]));  
        
        resampleImg0->SetInput(this->GetInput(0));
        resampleImg0->SetSize(outputSize);
        resampleImg0->SetOutputSpacing(outputSpacing);
        resampleImg0->UpdateLargestPossibleRegion();
        
        resampleImg1->SetInput(this->GetInput(1));
        resampleImg1->SetSize(outputSize);
        resampleImg1->SetOutputSpacing(outputSpacing);
        resampleImg1->UpdateLargestPossibleRegion();
        
        interpolatorIm1->SetInputImage(resampleImg1->GetOutput());
        
        itk::ResampleImageFilter<OutputImageType,OutputImageType>::Pointer resampleDu 
          = itk::ResampleImageFilter<OutputImageType,OutputImageType>::New();
        resampleDu->SetTransform(itk::IdentityTransform<double, 2>::New());
        resampleDu->SetInterpolator(interpolatorOut);    
        resampleDu->SetInput(auxDu);
        resampleDu->SetSize(outputSize);
        resampleDu->SetOutputSpacing(outputSpacing);
        resampleDu->UpdateLargestPossibleRegion();
        
        itk::ResampleImageFilter<OutputImageType,OutputImageType>::Pointer resampleDv 
          = itk::ResampleImageFilter<OutputImageType,OutputImageType>::New();
        resampleDv->SetTransform(itk::IdentityTransform<double, 2>::New());
        resampleDv->SetInterpolator(interpolatorOut);
        resampleDv->SetInput(auxDv);
        resampleDv->SetSize(outputSize);
        resampleDv->SetOutputSpacing(outputSpacing);
        resampleDv->UpdateLargestPossibleRegion();
        
        auxDu=resampleDu->GetOutput();
        auxDv=resampleDv->GetOutput();  
        
        //displacements
        itk::ImageRegionIterator< itk::Image<double, 2> > uIterator
          (auxDu,auxDu->GetLargestPossibleRegion());
        itk::ImageRegionIterator< itk::Image<double, 2> > vIterator
          (auxDv,auxDv->GetLargestPossibleRegion());
                
        while(!uIterator.IsAtEnd()){
          
          pixel[0] = (uIterator.Get())+uIterator.GetIndex()[0];
          pixel[1] = (vIterator.Get())+vIterator.GetIndex()[1];
          
          interpImg1->SetPixel(uIterator.GetIndex(),InputImageType::PixelType
            (interpolatorIm1->EvaluateAtContinuousIndex(pixel)) ); 
          
          ++uIterator;
          ++vIterator;
        }          
        
      }          
      
      //outputs Du and Dv
      itk::ImageRegionIterator< itk::Image<double, 2> > auxDuIterator
        (auxDu,auxDu->GetLargestPossibleRegion());
      itk::ImageRegionIterator< itk::Image<double, 2> > auxDvIterator
        (auxDv,auxDv->GetLargestPossibleRegion());
      itk::ImageRegionIterator< TOutputImage > outputIteratorU
        (this->GetNthOutput(0), this->GetNthOutput(0)->GetLargestPossibleRegion());
      itk::ImageRegionIterator< TOutputImage > outputIteratorV
        (this->GetNthOutput(1), this->GetNthOutput(1)->GetLargestPossibleRegion());
      
      while(!outputIteratorU.IsAtEnd()){
        outputIteratorU.Set(auxDuIterator.Get());
        outputIteratorV.Set(auxDvIterator.Get());
                
        ++outputIteratorU;
        ++outputIteratorV;
        ++auxDuIterator;
        ++auxDvIterator;
      }
    }
  }
  
  template< class TInputImage, class TOutputImage>
  void HermiteOpticalFlow<TInputImage, TOutputImage>::OpticalFlowHermite
    (const InputImageType* img0,
    const InputImageType* img1,itk::Image<double, 2>::Pointer Du,
    itk::Image<double, 2>::Pointer Dv)
  {       
///////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////parameters of functions////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////
    double epsilon = 0.01;//= 10^(-2)//variable for psi function
    int alpha = 50; //Variable of smooth regularization
    int gamma = 90; //Local constraint weight
    int beta = 50; //Homogeneity parameter
   
    //Sum of rotated coefficients
    itk::Image<double, 2>::IndexType regionStart;
      regionStart[0] = 0; 
      regionStart[1] = 0; 
      
    itk::Image<double, 2>::SizeType regionSize;
      regionSize = img0->GetLargestPossibleRegion().GetSize();
      
    itk::Image<double, 2>::RegionType region;
      region.SetSize( regionSize ); region.SetIndex( regionStart );
    
    // Du initial
    OutputImageType::Pointer U = OutputImageType::New();
    U->SetRegions(region);
    U->Allocate();
    U=Du;
  
    // Dv initial
    OutputImageType::Pointer V = OutputImageType::New();
    V->SetRegions(region);
    V->Allocate();
    V=Dv;
    
    // Du + Du initial
    OutputImageType::Pointer sDu = OutputImageType::New();
    sDu->SetRegions(region);
    sDu->Allocate();

    // Dv + Dv initial
    OutputImageType::Pointer sDv = OutputImageType::New();
    sDv->SetRegions(region);
    sDv->Allocate();
    
    itk::Image<float, 2>::Pointer I0 = itk::Image<float, 2>::New();
    I0->SetRegions(region);
    I0->Allocate();
    
    itk::Image<float, 2>::Pointer I1 = itk::Image<float, 2>::New();
    I1->SetRegions(region);
    I1->Allocate();
    
    itk::Image<float, 2>::Pointer I1m = itk::Image<float, 2>::New();
    I1m->SetRegions(region);
    I1m->Allocate();
    
    itk::Image<float, 2>::Pointer I1n = itk::Image<float, 2>::New();
    I1n->SetRegions(region);
    I1n->Allocate();    
    
    //Homogeneity Constraint
    itk::Image<double, 2>::Pointer homogeneity = itk::Image<double, 2>::New();
    homogeneity->SetRegions(region);
    homogeneity->Allocate();
    
    itk::ImageRegionIterator< itk::Image<double, 2> > homogeneityIterator
      (homogeneity, region);
    itk::ImageRegionIterator< itk::Image<float, 2> > I0Iterator(I0, region);
    itk::ImageRegionIterator< itk::Image<float, 2> > I1Iterator(I1, region);
    itk::ImageRegionIterator< itk::Image<float, 2> > I1mIterator(I1m, region);
    itk::ImageRegionIterator< itk::Image<float, 2> > I1nIterator(I1n, region); 
    
    //Coeffieciens of Hermite Transform of input images
    typedef itk::HermiteImageFilter<TInputImage, 
      itk::Image<float, 2> > HermiteFilterType;
        
    HermiteFilterType::Pointer hermiteFilter0 = HermiteFilterType::New();
    hermiteFilter0->SetInput( img0  );
    hermiteFilter0->Update();
    
    itk::ImageRegionConstIterator< itk::Image<float, 2> > h0c00Iterator
      (hermiteFilter0->GetCoef(0), 
      hermiteFilter0->GetCoef(0)->GetLargestPossibleRegion());
    itk::ImageRegionConstIterator< itk::Image<float, 2> > h0c10Iterator
      (hermiteFilter0->GetCoef(1), 
      hermiteFilter0->GetCoef(1)->GetLargestPossibleRegion());
    itk::ImageRegionConstIterator< itk::Image<float, 2> > h0c01Iterator
      (hermiteFilter0->GetCoef(2), 
      hermiteFilter0->GetCoef(2)->GetLargestPossibleRegion());
    itk::ImageRegionConstIterator< itk::Image<float, 2> > h0r10Iterator
      (hermiteFilter0->GetRotCoef(1), 
      hermiteFilter0->GetRotCoef(1)->GetLargestPossibleRegion());
    itk::ImageRegionConstIterator< itk::Image<float, 2> > h0r20Iterator
      (hermiteFilter0->GetRotCoef(2), 
      hermiteFilter0->GetRotCoef(2)->GetLargestPossibleRegion());
  
    HermiteFilterType::Pointer hermiteFilter1 = HermiteFilterType::New();
    hermiteFilter1->SetInput( img1  );
    hermiteFilter1->Update();
    
    itk::ImageRegionConstIterator< itk::Image<float, 2> > h1c00Iterator
      (hermiteFilter1->GetCoef(0), 
      hermiteFilter1->GetCoef(0)->GetLargestPossibleRegion());
    itk::ImageRegionConstIterator< itk::Image<float, 2> > h1c10Iterator
      (hermiteFilter1->GetCoef(1), 
      hermiteFilter1->GetCoef(1)->GetLargestPossibleRegion());
    itk::ImageRegionConstIterator< itk::Image<float, 2> > h1c01Iterator
      (hermiteFilter1->GetCoef(2), 
      hermiteFilter1->GetCoef(2)->GetLargestPossibleRegion());
    itk::ImageRegionConstIterator< itk::Image<float, 2> > h1c20Iterator
      (hermiteFilter1->GetCoef(3), 
      hermiteFilter1->GetCoef(3)->GetLargestPossibleRegion());
    itk::ImageRegionConstIterator< itk::Image<float, 2> > h1c11Iterator
      (hermiteFilter1->GetCoef(4), 
      hermiteFilter1->GetCoef(4)->GetLargestPossibleRegion());
    itk::ImageRegionConstIterator< itk::Image<float, 2> > h1c12Iterator
      (hermiteFilter1->GetCoef(8), 
      hermiteFilter1->GetCoef(8)->GetLargestPossibleRegion());
    itk::ImageRegionConstIterator< itk::Image<float, 2> > h1c21Iterator
      (hermiteFilter1->GetCoef(7), 
      hermiteFilter1->GetCoef(7)->GetLargestPossibleRegion());
    itk::ImageRegionConstIterator< itk::Image<float, 2> > h1c30Iterator
      (hermiteFilter1->GetCoef(6), 
      hermiteFilter1->GetCoef(6)->GetLargestPossibleRegion());
    itk::ImageRegionConstIterator< itk::Image<float, 2> > h1c02Iterator
      (hermiteFilter1->GetCoef(5), 
      hermiteFilter1->GetCoef(5)->GetLargestPossibleRegion());
    itk::ImageRegionConstIterator< itk::Image<float, 2> > h1c03Iterator
      (hermiteFilter1->GetCoef(9), 
      hermiteFilter1->GetCoef(9)->GetLargestPossibleRegion());
    itk::ImageRegionConstIterator< itk::Image<float, 2> > h1angleIterator
      (hermiteFilter1->GetAngle(), 
      hermiteFilter1->GetAngle()->GetLargestPossibleRegion());
    itk::ImageRegionConstIterator< itk::Image<float, 2> > h1r10Iterator
      (hermiteFilter1->GetRotCoef(1), 
      hermiteFilter1->GetRotCoef(1)->GetLargestPossibleRegion());
    itk::ImageRegionConstIterator< itk::Image<float, 2> > h1r20Iterator
      (hermiteFilter1->GetRotCoef(2), 
      hermiteFilter1->GetRotCoef(2)->GetLargestPossibleRegion());
               
    while(!homogeneityIterator.IsAtEnd()){
      //sum coefficients n,teta
    I0Iterator.Set(h0r10Iterator.Get() + h0r20Iterator.Get());
    I1Iterator.Set(h1r10Iterator.Get() + h1r20Iterator.Get());
    
    float sth = sin(h1angleIterator.Get());
    float cth = cos(h1angleIterator.Get());
    
    //sin and cos of rotation are same to rotation for 10
    float R1m1 = (h1c11Iterator.Get()*sth) + (h1c20Iterator.Get()*cth);
    
    //sin and cos of rotation are same to rotation for 20
    float R1m2 = (h1c12Iterator.Get()*sth*sth) + 
                 (h1c21Iterator.Get()*1.4142*cth*sth) + 
                 (h1c30Iterator.Get()*cth*cth);//fuang matlab  
    
    //sin and cos of rotation are same to rotation for 10
    float R1n1 = (h1c02Iterator.Get()*sth) + (h1c11Iterator.Get()*cth);
    
    //sin and cos of rotation are same to rotation for 20
    float R1n2 = (h1c03Iterator.Get()*sth*sth) + 
                 (h1c12Iterator.Get()*1.4142*cth*sth) + 
                 (h1c21Iterator.Get()*cth*cth);//fuang matlab
    
    //sum coefficients n,teta  (m)+1
    I1mIterator.Set(R1m1+R1m2);
    
    //sum coefficients n,teta  (n)+1
    I1nIterator.Set(R1n1+R1n2);
    
    homogeneityIterator.Set((1/( sqrt(pow(h1c10Iterator.Get()-
      h0c10Iterator.Get(),2) + pow(h1c01Iterator.Get()-
      h0c01Iterator.Get(),2) + pow(h1c00Iterator.Get()-
      h0c00Iterator.Get(),2))+vnl_math::eps ))*beta);  
    
    ++homogeneityIterator;
    ++I0Iterator;
    ++I1Iterator;
    ++I1mIterator;
    ++I1nIterator;
    
    ++h1angleIterator;
    
    ++h0c00Iterator;
    ++h1c00Iterator;
    ++h0c10Iterator;
    ++h1c10Iterator;
    ++h0c01Iterator;
    ++h1c01Iterator;
    ++h1c11Iterator;
    ++h1c12Iterator;
    ++h1c21Iterator;
    ++h1c02Iterator;
    ++h1c03Iterator;
    ++h1c20Iterator;
    ++h1c30Iterator;
    
    ++h0r10Iterator;
    ++h1r10Iterator;
    ++h0r20Iterator;
    ++h1r20Iterator;
    }
    
    itk::Image<double, 2>::Pointer psidash = itk::Image<double, 2>::New();
      psidash->SetRegions(region);
      psidash->Allocate();  
    
    itk::ImageRegionIterator< itk::Image<double, 2> > psidashIterator
      (psidash, psidash->GetLargestPossibleRegion());
    
    GradientX0 = itk::Image<double, 2>::New();
      GradientX0->SetRegions(region);
      GradientX0->Allocate();
        
    GradientY0 = itk::Image<double, 2>::New();    
      GradientY0->SetRegions(region);
      GradientY0->Allocate();
    
    GradientX1 = itk::Image<double, 2>::New();
      GradientX1->SetRegions(region);
      GradientX1->Allocate();
    
    GradientY1 = itk::Image<double, 2>::New();    
      GradientY1->SetRegions(region);
      GradientY1->Allocate();
        
    itk::Image<double, 2>::Pointer psidashFSx = itk::Image<double, 2>::New();
    itk::Image<double, 2>::Pointer psidashFSy = itk::Image<double, 2>::New();

    psidashFSx->SetRegions(region);
    psidashFSx->Allocate();
    
    psidashFSy->SetRegions(region);
    psidashFSy->Allocate();
    
    itk::ImageRegionIteratorWithIndex < itk::Image<double, 2> > 
      psidashFSxIterator(psidashFSx,region);
    itk::ImageRegionIteratorWithIndex < itk::Image<double, 2> > 
      psidashFSyIterator(psidashFSy,region);
         
    //rows of matrix A    
    itk::Array<unsigned int> rs;
    rs.SetSize(12*regionSize[0]*regionSize[1]);
    
    //columns of matrix A
    itk::Array<int> cs;
    cs.SetSize(12*regionSize[0]*regionSize[1]);
    
    //values of matrix A
    itk::Array<double> ss;
    ss.SetSize(12*regionSize[0]*regionSize[1]);
        
    //vector B   
    vnl_vector<double> vectb(2*regionSize[0]*regionSize[1]);
    
    //vector Duv, Group u and v in a only vector
    vnl_vector<double> vectDuv (2*regionSize[0]*regionSize[1]);    
       
    for(int io=0; io<itera_outer ; io++){  
      //std::cout<<"itera out = "<<io<<std::endl;
      homogeneityIterator.GoToBegin();
      h1c00Iterator.GoToBegin();
      h0c00Iterator.GoToBegin();
      h1c10Iterator.GoToBegin();
      h1c01Iterator.GoToBegin();
      I1Iterator.GoToBegin();
      I0Iterator.GoToBegin();
      I1mIterator.GoToBegin();
      I1nIterator.GoToBegin();
      
      itk::ImageRegionConstIterator< TOutputImage > UIterator
        (U, U->GetLargestPossibleRegion());
      itk::ImageRegionConstIterator< TOutputImage > VIterator
        (V, V->GetLargestPossibleRegion());
      itk::ImageRegionIteratorWithIndex< TOutputImage > duIterator
        (Du, Du->GetLargestPossibleRegion());
      itk::ImageRegionIteratorWithIndex< TOutputImage > dvIterator
        (Dv, Dv->GetLargestPossibleRegion());
      itk::ImageRegionIterator< TOutputImage > sDuIterator
        (sDu, sDu->GetLargestPossibleRegion());
      itk::ImageRegionIterator< TOutputImage > sDvIterator
        (sDv, sDv->GetLargestPossibleRegion());
      
      while(!sDuIterator.IsAtEnd()){

      //sum Du + Du initial
      sDuIterator.Set(duIterator.Get()+UIterator.Get());
      sDvIterator.Set(dvIterator.Get()+VIterator.Get());
      
      ++UIterator;
      ++VIterator;
      
      ++sDuIterator;
      ++sDvIterator;
      
      ++duIterator;
       ++dvIterator;
      }
      
      duIterator.GoToBegin();
      dvIterator.GoToBegin();
                  
      //gradient U
      GradientX0->FillBuffer(0.0);
      GradientY0->FillBuffer(0.0);
      
      Gradient(sDu,false);
     
      itk::ImageRegionIterator< itk::Image<double, 2> > GuxIterator
        (GradientX0,GradientX0->GetLargestPossibleRegion());
      itk::ImageRegionIterator< itk::Image<double, 2> > GuyIterator
        (GradientY0,GradientY0->GetLargestPossibleRegion());
      
      //gradient V
      GradientX1->FillBuffer(0.0);
      GradientY1->FillBuffer(0.0);
      
      Gradient(sDv,true);      
            
      itk::ImageRegionIterator< itk::Image<double, 2> > GvxIterator
        (GradientX1,GradientX1->GetLargestPossibleRegion());
      itk::ImageRegionIterator< itk::Image<double, 2> > GvyIterator
        (GradientY1,GradientY1->GetLargestPossibleRegion());
          
      unsigned int index = 0;
      
      //calculate psidash, psidashFSx and psidashFSy
      while(!psidashIterator.IsAtEnd()){
      double L = pow(h1c00Iterator.Get()-h0c00Iterator.Get()+(h1c10Iterator.Get()
              *duIterator.Get())+(h1c01Iterator.Get()*dvIterator.Get()),2)
        +gamma*pow(I1Iterator.Get()-I0Iterator.Get()+(I1mIterator.Get()
              *duIterator.Get())+(I1nIterator.Get()*dvIterator.Get()),2);
              
      psidashIterator.Set(1/(std::max(std::min((2.0*sqrt(L)),
                                               (1/epsilon)),epsilon)));
      
      psidashFSxIterator.Set(alpha*(1/(std::max(std::min((2.0*sqrt
        (GuxIterator.Get()+GvxIterator.Get())),(1/epsilon)),epsilon))));
      psidashFSyIterator.Set(alpha*(1/(std::max(std::min((2.0*sqrt
        (GuyIterator.Get()+GvyIterator.Get())),(1/epsilon)),epsilon))));
      
      ++GuxIterator;
      ++GuyIterator;
      ++GvxIterator;
      ++GvyIterator;
      
      ++psidashIterator;
      ++psidashFSxIterator;
      ++psidashFSyIterator;
      
      ++I0Iterator;
      ++I1Iterator;
      ++I1mIterator;
      ++I1nIterator;
      ++h0c00Iterator;
      ++h1c00Iterator;
      ++h1c10Iterator;
      ++h1c01Iterator;
      
      ++duIterator;
      ++dvIterator;
      }
      
      UIterator.GoToBegin();
      VIterator.GoToBegin();
      
      psidashIterator.GoToBegin();
      psidashFSxIterator.GoToBegin();
      psidashFSyIterator.GoToBegin();
      
      h1c00Iterator.GoToBegin();
      h0c00Iterator.GoToBegin();
      h1c10Iterator.GoToBegin();
      h1c01Iterator.GoToBegin();
      I1Iterator.GoToBegin();
      I0Iterator.GoToBegin();
      I1mIterator.GoToBegin();
      I1nIterator.GoToBegin();
      duIterator.GoToBegin();
      dvIterator.GoToBegin();
      
      //vector b
      vectb.fill(0.0);
      
      //vector duv
      vectDuv.fill(0);
            
      // psidashFSNext (index+1) cant calculte same time of psidashFS(index)
      while(!psidashIterator.IsAtEnd()){  
      double psidashFSxNext = 0.0;
      double psidashFSyNext = 0.0;
      
      itk::Image<double, 2>::IndexType psidashFSIndex = 
        psidashFSyIterator.GetIndex();
            
      if(psidashFSIndex[1]==0){//this row is 0
        psidashFSyIterator.Set(0);
      }  
                    
      if(psidashFSIndex[0]==0){//this column is 0
        psidashFSxIterator.Set(0);
      }
            
      psidashFSIndex = psidashFSyIterator.GetIndex();
      
      //dont exist pixel with indexY=imageDim[1]
      if(psidashFSIndex[1]<((img0->GetLargestPossibleRegion().GetSize()[1])-1)){
        ++psidashFSIndex[1];
        psidashFSyNext = psidashFSy->GetPixel(psidashFSIndex);
        --psidashFSIndex[1];
      }
          
      //dont exist pixel with indexX=imageDim[0]      
      if(psidashFSIndex[0]<((img0->GetLargestPossibleRegion().GetSize()[0])-1)){
        ++psidashFSIndex[0];
        psidashFSxNext = psidashFSx->GetPixel(psidashFSIndex);
        --psidashFSIndex[0];
      }
      
      double pdfsum = psidashFSxIterator.Get() + psidashFSxNext + 
        psidashFSyIterator.Get() + psidashFSyNext;
      
      //first Euler-Lagrange Ecuation for u
      double uapp = psidashIterator.Get()*((h1c10Iterator.Get()*
        h1c10Iterator.Get()) + gamma*((I1mIterator.Get()*
        I1mIterator.Get()))) + pdfsum + homogeneityIterator.Get();
            
      //first Euler-Lagrange Ecuation for v
      double uvapp = psidashIterator.Get()*((h1c10Iterator.Get()*
        h1c01Iterator.Get()) + gamma*(I1mIterator.Get()*
        I1nIterator.Get()));
      
      //second Euler-Lagrange Ecuation for u is same that the first Euler-Lagrange Ecuation for v
      /*
        double vuapp = psidashIterator.Get()*((h1c01Iterator.Get()*
          h1c10Iterator.Get()) + gamma*(I1nIterator.Get()*I1mIterator.Get()));
      */
      
      //second Euler-Lagrange Ecuation for v
      double vapp = psidashIterator.Get()*((h1c01Iterator.Get()*
        h1c01Iterator.Get()) + gamma*((I1nIterator.Get()*
        I1nIterator.Get()))) + pdfsum + homogeneityIterator.Get();
      
      double uPreviousX = 0;
      double vPreviousX = 0;
      double uPreviousY = 0;
      double vPreviousY = 0;
      double uNextX = 0;
      double vNextX = 0;
      double uNextY = 0;
      double vNextY = 0;
      
      //previous and next u,v ; u and v have same indexs
      itk::Image<double, 2>::IndexType uvIndex;    
      uvIndex[0] = duIterator.GetIndex()[0];
      
      //dont exist pixel with indexY=-1 
      if( (duIterator.GetIndex()[1]) > 0 ){
        uvIndex[1] = (duIterator.GetIndex()[1])-1;
        uPreviousY = Du->GetPixel(uvIndex);
        vPreviousY = Dv->GetPixel(uvIndex);                
      }
      
      //dont exist pixel with indexY=imageDim[1]
      if((duIterator.GetIndex()[1]) < 
         ((img0->GetLargestPossibleRegion().GetSize()[1])-1)){
        uvIndex[1] = (duIterator.GetIndex()[1])+1;
        uNextY = Du->GetPixel(uvIndex);
        vNextY = Dv->GetPixel(uvIndex);
      }
      
      //dont exist pixel with indexX=-1
      uvIndex[1] = duIterator.GetIndex()[1];
      if((duIterator.GetIndex()[0]) > 0){
        uvIndex[0] = (duIterator.GetIndex()[0])-1;
        uPreviousX = Du->GetPixel(uvIndex);
        vPreviousX = Dv->GetPixel(uvIndex);        
      }
      
      //dont exist pixel with indexX=imageDim[0]
      if((duIterator.GetIndex()[0]) < 
         ((img0->GetLargestPossibleRegion().GetSize()[0])-1)){        
        uvIndex[0] = (duIterator.GetIndex()[0])+1;
        uNextX = Du->GetPixel(uvIndex);
        vNextX = Dv->GetPixel(uvIndex);        
      }
      
      //Constant terms for first Euler-Lagrange equation
      double pdfaltsumuU = psidashFSxIterator.Get() * 
        (uPreviousX-duIterator.Get()) + psidashFSxNext*
        (uNextX-duIterator.Get()) + psidashFSyIterator.Get() * 
        (uPreviousY-duIterator.Get()) + psidashFSyNext * 
        (uNextY-duIterator.Get());
      
      //Constant terms for second Euler-Lagrange equation
      double pdfaltsumuV = psidashFSxIterator.Get() * 
        (vPreviousX-dvIterator.Get()) + psidashFSxNext *
        (vNextX-dvIterator.Get()) + psidashFSyIterator.Get() * 
        (vPreviousY-dvIterator.Get()) +  psidashFSyNext * 
        (vNextY-dvIterator.Get());
            
      //iterators run over rows but we nead run over columns
      int pos = (h0c00Iterator.GetIndex()[0]*regionSize[1])+
	    h0c00Iterator.GetIndex()[1];
      
      //vector b = (u0,v0,u1,v1,...)      
      //pos. u in vector b= 2*pos
      vectb( 2*pos )= -(psidashIterator.Get()*((h1c10Iterator.Get() *
        (h1c00Iterator.Get()-h0c00Iterator.Get())) +gamma *
        (I1mIterator.Get()*(I1Iterator.Get()-I0Iterator.Get()))) -
        pdfaltsumuU);      
      //pos. v in vector b= (2*pos)+1
      vectb( (2*pos)+1 )= -(psidashIterator.Get()*((h1c01Iterator.Get() *
        (h1c00Iterator.Get()-h0c00Iterator.Get())) +
        gamma*(I1nIterator.Get()*(I1Iterator.Get()-I0Iterator.Get()))) -
        pdfaltsumuV);
      
      //rows of A = [0,0,0,0,0,0,1,1,1,1,1,1, ... ,(2*regionSize[0]*regionSize[1])-1 ]
      for(int aux=0; aux<12 ; aux++){
        rs[(12*index)+aux]=vtkMath::Floor(((12*index)+aux)/6);  
      }          
      
      //columns of A,regionSize[1]=rows in image
      cs[12*index]=rs.GetElement(12*index)-(2* regionSize[1]);//x-1
      cs[(12*index)+6]=rs.GetElement((12*index)+6)-(2* regionSize[1]);//x-1
      cs[(12*index)+1]=rs.GetElement((12*index)+1)-2;//y-1
      cs[(12*index)+7]=rs.GetElement((12*index)+7)-2;//y-1
      cs[(12*index)+2]=rs.GetElement((12*index)+2);//u
      cs[(12*index)+8]=rs.GetElement((12*index)+8)-1;//u
      cs[(12*index)+3]=rs.GetElement((12*index)+3)+1;//v
      cs[(12*index)+9]=rs.GetElement((12*index)+9);//v
      cs[(12*index)+4]=rs.GetElement((12*index)+4)+2;//y+1
      cs[(12*index)+10]=rs.GetElement((12*index)+10)+2;//y+1
      cs[(12*index)+5]=rs.GetElement((12*index)+5)+(2* regionSize[1]);//x+1
      cs[(12*index)+11]=rs.GetElement((12*index)+11)+(2* regionSize[1]);//x+1
            
      //values of A
      ss[12*pos]=-psidashFSxIterator.Get();
      ss[(12*pos)+6]=-psidashFSxIterator.Get();
      ss[(12*pos)+1]=-psidashFSyIterator.Get();
      ss[(12*pos)+7]=-psidashFSyIterator.Get();
      ss[(12*pos)+2]=uapp;
      ss[(12*pos)+8]=uvapp;
      ss[(12*pos)+3]=uvapp;
      ss[(12*pos)+9]=vapp;
      ss[(12*pos)+4]=-psidashFSyNext;
      ss[(12*pos)+10]=-psidashFSyNext;
      ss[(12*pos)+5]=-psidashFSxNext;
      ss[(12*pos)+11]=-psidashFSxNext;
      
      //vector duv for SOR method
      vectDuv(2*index) = duIterator.Get();//u
      vectDuv((2*index)+1) = dvIterator.Get();//v  
                          
      index++;
      
      ++homogeneityIterator;
      
      ++psidashIterator;
      ++psidashFSxIterator;
      ++psidashFSyIterator;
      
      ++I0Iterator;
      ++I1Iterator;
      ++I1mIterator;
      ++I1nIterator;
      ++h0c00Iterator;
      ++h1c00Iterator;
      ++h1c10Iterator;
      ++h1c01Iterator;
      
      ++duIterator;
      ++dvIterator;
      
      }
      psidashIterator.GoToBegin();
      psidashFSxIterator.GoToBegin();
      psidashFSyIterator.GoToBegin();
      duIterator.GoToBegin();
      dvIterator.GoToBegin();
            
      sorDu = itk::Image<double, 2>::New();
      sorDu->SetRegions(region);
      sorDu->Allocate();
      sorDu->FillBuffer(0.0);
      
      sorDv = itk::Image<double, 2>::New();
      sorDv->SetRegions(region);
      sorDv->Allocate();
      sorDv->FillBuffer(0.0);
            
      SOR(rs,cs,ss,vectb,vectDuv);
      
      Du = sorDu;
      Dv = sorDv;
      
      Du->Update();
      Dv->Update();
      
    }//external iteration end
        
  }
  
  template< class TInputImage, class TOutputImage>
  DataObject::Pointer HermiteOpticalFlow<TInputImage, TOutputImage>::MakeOutput()
  {
    typename TOutputImage::Pointer output;
    output = ( TOutputImage::New() ).GetPointer();
    return output.GetPointer();
  }
  
  template< class TInputImage, class TOutputImage>
  void HermiteOpticalFlow<TInputImage, TOutputImage>::Gradient
    (itk::Image<double, 2>::Pointer imgD, bool flag)
  {    
    double pix = 0;
    double x = 0;  
    double y = 0;
    double Xaux = 0;
    double xd = 0;
    double yd = 0;  
    double txd = 0;  
    double tyd = 0;
    
    double MC0 = 0;  
    double MC1 = 0;
    double MC2 = 0;
    
    double valX = 0.0;
    double valY = 0.0;
    
    std::vector< double >MR0
      ((imgD->GetLargestPossibleRegion().GetSize()[1])+4);
    std::vector< double >MR1
      ((imgD->GetLargestPossibleRegion().GetSize()[1])+4);
    std::vector< double >MR2
      ((imgD->GetLargestPossibleRegion().GetSize()[1])+4);
    std::vector< double >Yaux
      ((imgD->GetLargestPossibleRegion().GetSize()[1])+4);
    
    for(int k=0;k<(4+imgD->GetLargestPossibleRegion().GetSize()[1]);k++){
      MR0[k] = 0.0;
      MR1[k] = 0.0;
      MR2[k] = 0.0;
      Yaux[k] = 0.0;
    }
    
    itk::Image<double, 2>::IndexType pixelIndex;
    
    for(int i=0;i<(4+imgD->GetLargestPossibleRegion().GetSize()[0]);i++){
    for(int j=0;j<(4+imgD->GetLargestPossibleRegion().GetSize()[1]);j++){
      //index of pixel
      pixelIndex[0] = i-2;  
      pixelIndex[1] = j-2;
      
      //pixelValue
      if(  (pixelIndex[0]<0) || (pixelIndex[1]<0) ||
        (pixelIndex[0]>(imgD->GetLargestPossibleRegion().GetSize()[0]-1)) ||
        (pixelIndex[1]>(imgD->GetLargestPossibleRegion().GetSize()[1]-1))) {      
        //zero-pading
        pix = 0;
      }else{
        pix = ( imgD->GetPixel(pixelIndex) );
      }

      x = pix - MR0[j];
      MR0[j] =pix;
      
      y = pix - MC0;
      MC0 = pix;
            
      xd = (x + MR1[j])/2;
      MR1[j] = x;
      
      yd = (y + MC1)/2;
      MC1 = y;
      
      txd = (xd + MC2)/2;
      MC2 = xd;
      
      tyd = (yd + MR2[j])/2;
      MR2[j] = yd;
      
      //out of phase sum
      if(pixelIndex[0]>=1){
        valY = (Yaux[j]*Yaux[j])+(txd*txd);
      }
      if(pixelIndex[1]>=1){
        valX = (Xaux*Xaux)+(tyd*tyd);
      }
      if(pixelIndex[0]>=0){
        Yaux[j] = y;
      }
      if(pixelIndex[1]>=0){
        Xaux = x;
      }
      
      //cut for Gx
      if((pixelIndex[0]>=1) &
         (pixelIndex[1]>=1) &
         (pixelIndex[0]<imgD->GetLargestPossibleRegion().GetSize()[0]) &
         (pixelIndex[1]<=imgD->GetLargestPossibleRegion().GetSize()[1])) {  
            
        pixelIndex[1]=pixelIndex[1]-1;
        
        if(flag==false){
          GradientX0->SetPixel( pixelIndex,valX ); 
        }else{      
          GradientX1->SetPixel( pixelIndex,valX );   
        }
        pixelIndex[1]=pixelIndex[1]+1;
      }      
      //cut for Gy
      if((pixelIndex[0]>=1) &
         (pixelIndex[1]>=1) &
         (pixelIndex[0]<=imgD->GetLargestPossibleRegion().GetSize()[0]) &
         (pixelIndex[1]<imgD->GetLargestPossibleRegion().GetSize()[1])) {    
          
        pixelIndex[0]=pixelIndex[0]-1;
        
        if(flag==false){
          GradientY0->SetPixel( pixelIndex,valY );         
        }else{
          GradientY1->SetPixel( pixelIndex,valY );         
        }
      }      
    }
    }
  }
  
  template< class TInputImage, class TOutputImage>
  void HermiteOpticalFlow<TInputImage, TOutputImage>::SOR
    (itk::Array<unsigned int> rs,itk::Array<int> cs,itk::Array<double> ss, 
      vnl_vector<double> b,vnl_vector<double> duv)
  {  
///////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////internal iterations////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////
    int itera_iner = 30;
    double tol = 1/pow(10.0,8); //= 10^(-8)
    float w = 1.8;//relaxation parameter
        
    unsigned int itera = 0;
    
    vnl_vector< double > vnlB(duv.size());
    vnl_vector< double > newX(duv.size());
    vnl_vector< double > oldX(duv.size());
    vnl_vector< double > aux(duv.size());
    
    //vectors
    for(int i=0;i<duv.size();i++){  
      vnlB(i)=b(i)*w;  
    }  
    
    newX=duv;
    oldX=duv;      
    
    //sparse matrix A(cs.GetElement(i),rs.GetElement(i)) = ss.GetElement(i);  
    
    vnl_sparse_matrix<double> M(duv.size(),duv.size());
    vnl_sparse_matrix<double> N(duv.size(),duv.size());
    
    for(int i=0; i<6*duv.size() ; i++){
      if((cs.GetElement(i)>=0)&(cs.GetElement(i)<duv.size())&
         (ss.GetElement(i)!=0)){
        if(cs.GetElement(i)==rs.GetElement(i)){
        //(i==j)==diag 
          //M(i,i) = A(i,i)
          M(cs.GetElement(i),rs.GetElement(i)) = ss.GetElement(i);
          //N(i,i) = (1.0-w)*A(i,i)
          N(cs.GetElement(i),rs.GetElement(i)) = (1.0-w)*ss.GetElement(i);
        }else{
          if(rs.GetElement(i)>cs.GetElement(i)){
          //(j>i)==upper triangular components
            //N(i,j)= -w*A(i,j)
            N(cs.GetElement(i),rs.GetElement(i)) = -w*ss.GetElement(i);
          }else{
          //(j<i)==lower  triangular components
            //M(i,j)= w*A(i,j)
            M(cs.GetElement(i),rs.GetElement(i)) = w*ss.GetElement(i);
          }
        }      
      }  
    }
    
    vnl_sparse_lu linearSolver(M, vnl_sparse_lu::estimate_condition);
    float error;
    
    while(itera < itera_iner){
      //aux = N * duv
      N.mult(oldX,aux);
      
      aux = vnlB + aux;      
    
      //calculate newX of M*newX = N*x + w*b
      linearSolver.solve(aux,&newX);
        
      //stop condition
      aux = newX-oldX;
      error = aux.two_norm()/newX.two_norm();
      
      if(error<=tol){
        //converge, stop procedure
        //std::cout<<"converge"<<std::endl;
        itera = itera_iner;
      }else{
        //not converge, continue procedure with the new solution
        itera++;  
      }
      
      //actialize oldX for new iteration
      oldX = newX;      
    }
    
    //separate newDuv     
    itk::ImageRegionIterator< itk::Image<double, 2> > SORuIterator
      (sorDu,sorDu->GetLargestPossibleRegion());
    itk::ImageRegionIterator< itk::Image<double, 2> > SORvIterator
      (sorDv,sorDv->GetLargestPossibleRegion()); 
  
    while(!SORuIterator.IsAtEnd()){        
      int pos = (SORuIterator.GetIndex()[0] * 
        (sorDu->GetLargestPossibleRegion().GetSize()[1])) + 
        SORuIterator.GetIndex()[1];
      
      SORuIterator.Set(newX(2*pos));
      SORvIterator.Set(newX((2*pos)+1));
      
      ++SORuIterator;
      ++SORvIterator;
    }    
  }
  
  template< class TInputImage, class TOutputImage>
  TOutputImage* HermiteOpticalFlow<TInputImage, TOutputImage>::
    GetNthOutput(int n)
  {
    return dynamic_cast< TOutputImage * >( this->ProcessObject::
      GetOutput(n) );
  }
  
  template< class TInputImage, class TOutputImage>
  TOutputImage* HermiteOpticalFlow<TInputImage, TOutputImage>::GetDu()
  {
    return GetNthOutput(0);
  }

  template< class TInputImage, class TOutputImage>
  TOutputImage* HermiteOpticalFlow<TInputImage, TOutputImage>::GetDv()
  {
    return GetNthOutput(1);
  }
 
}// end namespace 
 
#endif