/*
 * File:   Optical Flow
 * Author: Fabiola Pacheco
 * 
 * Created on 08/18/2017
 * Last modification on 10/05/2017
 */
#include "opticalflowwidget.h"
#include "ui_opticalflowwidget.h"
#include <QVTKWidget.h>
#include "mainwindow.h"

#include "ofsegwidget.h"
#include "ui_ofsegwidget.h"

#include <QVBoxLayout>

#include <vtkParametricSpline.h>
#include <vtkKochanekSpline.h>
#include <vtkParametricFunctionSource.h>
#include <vtkGlyphSource2D.h>
#include <vtkImageShiftScale.h>
#include "vtkTextProperty.h"
#include <vtkDoubleArray.h>
#include "vtkContextScene.h"
#include "vtkContextItem.h"
#include "vtkContext2D.h"
#include <vtkTable.h>
#include <vtkAxis.h>
#include <vtkBrush.h>
#include <vtkRect.h>

#include "QVTKImageWidgetCommand.h"

OpticalFlowWidget::OpticalFlowWidget(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::OpticalFlowWidget)
{
    ui->setupUi(this);

  //initializing optical flow display widget
  imageOFwidget = new QVTKWidget;
  QVBoxLayout *ofLayout = new QVBoxLayout;
  ofLayout->setContentsMargins(0, 0, 0, 0);
  ofLayout->setSpacing(0);
  ofLayout->QLayout::addWidget(imageOFwidget);
  ui->imageOFwidget->setLayout(ofLayout);

  //initializing M-Mode display widget
  mModewidget = new QVTKWidget;
  QVBoxLayout *mModeLayout = new QVBoxLayout;
  mModeLayout->setContentsMargins(0, 0, 0, 0);
  mModeLayout->setSpacing(0);
  mModeLayout->QLayout::addWidget(mModewidget);
  ui->mModewidget->setLayout(mModeLayout);

}

OpticalFlowWidget::~OpticalFlowWidget()
{
    delete ui;
}

void OpticalFlowWidget::setOpticalFlowData
  (std::vector< vtkSmartPointer<vtkImageData> > images,
   std::vector< itk::Image<double,2>::Pointer > OFduITK,
   std::vector< itk::Image<double,2>::Pointer > OFdvITK) 
{
  imgVTK = images;
  duITK = OFduITK;
  dvITK = OFdvITK;
  
  mModeFlag = false;
  mModeCicles = 1;

  scale = ui->ScaleBox->value();

  frame = 0;
  
  //initializing 2D view of Optical Flow
  imageStyleImage = vtkSmartPointer<vtkInteractorStyleImage>::New();
  viewerImage = vtkSmartPointer<vtkImageViewer2>::New();
  imageActor = vtkSmartPointer<vtkImageActor>::New();  
  imageOFwidget->SetRenderWindow(viewerImage->GetRenderWindow());
  imageActor = viewerImage->GetImageActor();
  imageActor->InterpolateOff();
  viewerImage->GetRenderWindow()->GetInteractor()->
	  SetInteractorStyle(imageStyleImage);

  //initialize renderer for M-Mode widget
  rendererMmode = vtkSmartPointer<vtkRenderer>::New();
  mModewidget->GetRenderWindow()->AddRenderer(rendererMmode);

  //initialize graphic 
  chartXY = vtkSmartPointer<vtkChartXY>::New();
  chartXY->AutoAxesOn();

  chartXY->GetAxis(vtkAxis::BOTTOM)->SetTitle("Secuence");
  chartXY->GetAxis(vtkAxis::BOTTOM)->GetTitleProperties()->
	  SetColor(1,1,1);
  chartXY->GetAxis(vtkAxis::BOTTOM)->GetLabelProperties()->
	  SetColor(1,1,1);

  chartXY->GetAxis(vtkAxis::LEFT)->SetTitle("Magnitude");
  chartXY->GetAxis(vtkAxis::LEFT)->GetTitleProperties()->
	  SetColor(1,1,1);
  chartXY->GetAxis(vtkAxis::LEFT)->GetLabelProperties()->
	  SetColor(1,1,1);

  chartXY->SetAutoSize(true);

  contextMmodeActor = vtkSmartPointer<vtkContextActor>::New();
  contextMmodeActor->GetScene()->AddItem(chartXY);
  contextMmodeActor->GetScene()->SetRenderer(rendererMmode);
    
  //range of Slider
  ui->sequenceSlider->setRange(0,imgVTK.size()-1);

  //disable M-Mode elements
  ui->invertColorBox->setDisabled(true);
  ui->invertImageBox->setDisabled(true);
  ui->periodBeginSld->setDisabled(true);
  ui->periodEndSld->setDisabled(true);

  //Display first Image with vectors and scalar bar
  createAllVectors();
  scalarBarActor = allScalarBarActors[0];
  vectorActor = allVectorActors[0];
  viewerImage->SetInput(images[0]);
  viewerImage->GetRenderer()->AddViewProp(vectorActor);
  viewerImage->GetRenderer()->AddViewProp(scalarBarActor);

  viewerImage->Render();  
}

void OpticalFlowWidget::setMmodeData(vtkSmartPointer<vtkImageData> mModeImage,
									 int period[2],int cicles) 
{
  mMode = mModeImage;
  mMode2 = mModeImage;
  mModeFlipY = mModeImage;

  mModeCicles = cicles;
  mModePeriod[0] = period[0];
  mModePeriod[1] = period[1];
      
  mModeFlag = true;
  
  //initializing 2D view of M-Mode and graphics
  imageStyleImageMmode = vtkSmartPointer<vtkInteractorStyleImage>::New();
  mModeMapper = vtkSmartPointer<vtkImageSliceMapper>::New();
  mModeSlice = vtkSmartPointer<vtkImageSlice>::New();
  rendererMmode->GetRenderWindow()->GetInteractor()->
    SetInteractorStyle(imageStyleImageMmode);  
    
  //initialize visual reference of position M-Mode
  imageRefLineView = vtkSmartPointer<vtkLineSource>::New();
  imageRefLineViewMapper = vtkSmartPointer<vtkPolyDataMapper>::New();
  imageRefLineViewActor = vtkSmartPointer<vtkActor>::New();
  imageRefLineViewActor->GetProperty()->SetLineWidth(1);
  imageRefLineViewActor->GetProperty()->SetColor(255,50,0);
  imageRefLineViewMapper->SetInputConnection(imageRefLineView->
	  GetOutputPort());
  imageRefLineViewActor->SetMapper(imageRefLineViewMapper);

  //initialize visual reference of period
  imageRefLineViewB = vtkSmartPointer<vtkLineSource>::New();
  imageRefLineViewE = vtkSmartPointer<vtkLineSource>::New();
  imageRefLineViewMapperB = vtkSmartPointer<vtkPolyDataMapper>::New();
  imageRefLineViewMapperE = vtkSmartPointer<vtkPolyDataMapper>::New();
  imageRefLineViewActorB = vtkSmartPointer<vtkActor>::New();
  imageRefLineViewActorE = vtkSmartPointer<vtkActor>::New();

  imageRefLineViewActorB->GetProperty()->SetLineWidth(1);
  imageRefLineViewActorE->GetProperty()->SetLineWidth(1);
  imageRefLineViewActorB->GetProperty()->SetColor(220,0,0);
  imageRefLineViewActorE->GetProperty()->SetColor(220,0,0);
  imageRefLineViewMapperB->SetInputConnection(imageRefLineViewB->
	  GetOutputPort());
  imageRefLineViewMapperE->SetInputConnection(imageRefLineViewE->
	  GetOutputPort());
  imageRefLineViewActorB->SetMapper(imageRefLineViewMapperB);
  imageRefLineViewActorE->SetMapper(imageRefLineViewMapperE);

  //initial position of visual references
  imageRefLineView->SetPoint1(0,0,0);
  imageRefLineView->SetPoint2(0,
	  mMode2->GetDimensions()[1]*mMode2->GetSpacing()[1],0);
  imageRefLineView->Update();
  imageRefLineViewMapper->Update();
  rendererMmode->AddActor(imageRefLineViewActor);

  imageRefLineViewB->SetPoint1(period[0]*mMode2->GetSpacing()[0],0,0);
  imageRefLineViewB->SetPoint2(period[0]*mMode2->GetSpacing()[0],
    mMode2->GetDimensions()[1]*mMode2->GetSpacing()[1],0);
  imageRefLineViewB->Update();
  imageRefLineViewMapperB->Update();
  rendererMmode->AddActor(imageRefLineViewActorB);

  imageRefLineViewE->SetPoint1(period[1]*mMode2->GetSpacing()[0],0,0);
  imageRefLineViewE->SetPoint2(period[1]*mMode2->GetSpacing()[0],
    mMode2->GetDimensions()[1]*mMode2->GetSpacing()[1],0);
  imageRefLineViewE->Update();  
  imageRefLineViewMapperE->Update();
  rendererMmode->AddActor(imageRefLineViewActorE);

  //enable M-Mode elements
  ui->invertColorBox->setEnabled(true);
  ui->invertImageBox->setEnabled(true);
  ui->periodBeginSld->setEnabled(true);
  ui->periodEndSld->setEnabled(true);

  //range of Sliders
  ui->sequenceSlider->setRange(0, 
	  (mModeCicles*mMode->GetDimensions()[0])-1);
  ui->periodBeginSld->setRange(0, 
	  (mModeCicles*mMode->GetDimensions()[0])-1);
  ui->periodEndSld->setRange(0, 
	  (mModeCicles*mMode->GetDimensions()[0])-1);
  
  //set references to Sliders
  ui->periodBeginSld->setSliderPosition(period[0]);
  ui->periodEndSld->setSliderPosition(period[1]);

  //set image M-Mode
  ui->repeatBox->setValue(cicles);
  setCicles(cicles);
  mModeMapper->SetInputConnection(mMode2->GetProducerPort());

  mModeSlice->SetMapper(mModeMapper);
 
  rendererMmode->AddViewProp(mModeSlice);
  
  rendererMmode->ResetCamera();
  mModewidget->GetRenderWindow()->Render();
  
}

void OpticalFlowWidget::viewOFBoxClicked(bool value){    
  if(value){
    ui->ScaleBox->setEnabled(true);

    //actualize  actor
    if(ui->onSegmentRBtn->isChecked()){
      scalarBarActor = segScalarBarActors[frame];
      vectorActor = segVectorActors[frame];

      viewerImage->GetRenderer()->RemoveViewProp(segmentActor);
      segmentActor = segSplineActors[frame];
      viewerImage->GetRenderer()->AddViewProp(segmentActor);

    }else{
      scalarBarActor = allScalarBarActors[frame];
      vectorActor = allVectorActors[frame];
    }
    //Display vectors and scalar bar
    viewerImage->GetRenderer()->AddViewProp(vectorActor);
    viewerImage->GetRenderer()->AddViewProp(scalarBarActor);
  }else{
    ui->ScaleBox->setDisabled(true);
    
    //remove vectors and scalar bar     
    viewerImage->GetRenderer()->RemoveViewProp(vectorActor);
    viewerImage->GetRenderer()->RemoveViewProp(scalarBarActor);
    viewerImage->GetRenderer()->RemoveViewProp(segmentActor);
  }
  viewerImage->Render();  
}

void OpticalFlowWidget::vectSegRBtnClicked(bool value){  
  
  if(value & ui->onSegmentRBtn->isEnabled()){    
    if(ui->viewOFBox->isChecked()){
      //remove old vectors and scalar bar     
      viewerImage->GetRenderer()->RemoveViewProp(vectorActor);
      viewerImage->GetRenderer()->RemoveViewProp(scalarBarActor);
      viewerImage->GetRenderer()->RemoveViewProp(segmentActor);

      scalarBarActor = segScalarBarActors[frame];
      vectorActor = segVectorActors[frame];
      segmentActor = segSplineActors[frame];
    
      if(ui->segmentBtn->isEnabled()){
        //Display new vectors and scalar bar
        viewerImage->GetRenderer()->AddViewProp(vectorActor);
        viewerImage->GetRenderer()->AddViewProp(scalarBarActor);
        viewerImage->GetRenderer()->AddViewProp(segmentActor);

        viewerImage->Render();
      }
    }else{
      scalarBarActor = segScalarBarActors[frame];
      vectorActor = segVectorActors[frame];
    }    
  }else{    
    if(ui->viewOFBox->isChecked()){
      //remove old vectors and scalar bar     
      viewerImage->GetRenderer()->RemoveViewProp(vectorActor);
      viewerImage->GetRenderer()->RemoveViewProp(scalarBarActor);
      viewerImage->GetRenderer()->RemoveViewProp(segmentActor);

      scalarBarActor = allScalarBarActors[frame];
      vectorActor = allVectorActors[frame];

      if(ui->segmentBtn->isEnabled()){
        //Display new vectors and scalar bar
        viewerImage->GetRenderer()->AddViewProp(vectorActor);
        viewerImage->GetRenderer()->AddViewProp(scalarBarActor);

        viewerImage->Render();
      }
    }else{
      scalarBarActor = allScalarBarActors[frame];
      vectorActor = allVectorActors[frame];
    }    
  }    
  
}



void OpticalFlowWidget::changeDisplayedSeq(int seq)
{
  ui->sequenceSlider->update();

  frame =  seq - vtkMath::Floor(seq/imgVTK.size())*(imgVTK.size());

  //Display image
  viewerImage->SetInput(imgVTK[frame]);

  //if exists, move M-Modes reference
  if(mModeFlag){    
    if(ui->graphicBox->isChecked()==false){
      //remove last references
      rendererMmode->RemoveActor(imageRefLineViewActor);

      //position of new reference
      imageRefLineView->SetPoint1(seq*mMode2->GetSpacing()[0],0,0);
      imageRefLineView->SetPoint2(seq*mMode2->GetSpacing()[0],
      mMode2->GetDimensions()[1]*mMode2->GetSpacing()[1],0);
      
        
      imageRefLineView->Update();
        
      imageRefLineViewMapper->Update();  
    
      rendererMmode->AddActor(imageRefLineViewActor);
      mModewidget->GetRenderWindow()->Render();
    }
  }


  if(ui->viewOFBox->isChecked()){
    //remove old vectors and scalar bar     
    viewerImage->GetRenderer()->RemoveViewProp(vectorActor);
    viewerImage->GetRenderer()->RemoveViewProp(scalarBarActor);
    viewerImage->GetRenderer()->RemoveViewProp(segmentActor);

    //actualize vectors actors
    if(ui->onSegmentRBtn->isChecked()){
      scalarBarActor = segScalarBarActors[frame];
      vectorActor = segVectorActors[frame];
      segmentActor = segSplineActors[frame];
      viewerImage->GetRenderer()->AddViewProp(segmentActor);
    }else{
      scalarBarActor = allScalarBarActors[frame];
      vectorActor = allVectorActors[frame];
    }

    //Display new vectors and scalar bar
    viewerImage->GetRenderer()->AddViewProp(vectorActor);
    viewerImage->GetRenderer()->AddViewProp(scalarBarActor);

  }else{
    //remove vectors and scalar bar     
    viewerImage->GetRenderer()->RemoveViewProp(vectorActor);
    viewerImage->GetRenderer()->RemoveViewProp(scalarBarActor);
    viewerImage->GetRenderer()->RemoveViewProp(segmentActor);
  }

  viewerImage->Render();  

}

void OpticalFlowWidget::setCicles(int num){

  if(mModeFlag){

    if((mModePeriod[0]<(num*mMode->GetDimensions()[0]))&
     (mModePeriod[1]<(num*mMode->GetDimensions()[0]))){
      //actualize mModeCicles
      mModeCicles = num;

      //resize range of Slider
      ui->sequenceSlider->setRange(0,(num*mMode->GetDimensions()[0])-1);
      ui->periodBeginSld->setRange(0,(num*mMode->GetDimensions()[0])-1);
      ui->periodEndSld->setRange(0,(num*mMode->GetDimensions()[0])-1);

      //resize M-mode2 for save "num" cicles
      mMode2 = vtkSmartPointer<vtkImageData>::New();
      mMode2->SetOrigin( mMode->GetOrigin() );
      mMode2->SetDimensions( mMode->GetDimensions()[0]*num , 
		  mMode->GetDimensions()[1],1 );
      mMode2->SetSpacing(mMode->GetSpacing());
      mMode2->SetScalarTypeToUnsignedChar();
      mMode2->SetNumberOfScalarComponents(1);
      mMode2->Update();

      mModeFlipY = vtkSmartPointer<vtkImageData>::New();
      mModeFlipY->SetOrigin( mMode->GetOrigin() );
      mModeFlipY->SetDimensions( mMode->GetDimensions()[0]*num ,
		  mMode->GetDimensions()[1],1 );  
      mModeFlipY->SetSpacing(mMode->GetSpacing());
      mModeFlipY->SetScalarTypeToUnsignedChar();
      mModeFlipY->SetNumberOfScalarComponents(1);
      mModeFlipY->Update();

      //fill M-Mode2 and  flip M-Mode2
      for(int x = 0; x < mMode->GetDimensions()[0]; x++){
        for(int y = 0; y < mMode->GetDimensions()[1]; y++){
          for(int n = 0; n < num; n++){
            unsigned char * valueMmode = static_cast<unsigned char *> 
        ( mMode2->GetScalarPointer((n*(mMode->GetDimensions()[0]))+x,y,0) );
            unsigned char * valueFlipMmode = static_cast<unsigned char *> 
        ( mModeFlipY->GetScalarPointer((n*(mMode->GetDimensions()[0]))+
          x,mMode->GetDimensions()[1]-1-y,0) );
            valueMmode[0] = static_cast<unsigned char *> 
        ( mMode->GetScalarPointer(x,y,0) )[0];
            valueFlipMmode[0] = static_cast<unsigned char *> 
        ( mMode->GetScalarPointer(x,y,0) )[0];
          }
        }
      }  

      mMode2->Update();
      mModeFlipY->Update();    
    }else{
      //dont change cicles
      num = mModeCicles;
      ui->repeatBox->setValue(num);    
    }
    
    if(ui->graphicBox->isChecked()){
      //std::cout<<"cicles = "<<num<<std::endl;
      chartXY->ClearPlots();
      fillPlot();    

      if(ui->invertImageBox->isChecked()){  
        if(ui->invertColorBox->isChecked()){
          //invert intensity
          vtkSmartPointer<vtkImageShiftScale> inverter = 
        vtkSmartPointer<vtkImageShiftScale>::New();
          inverter->SetInput(mModeFlipY);
          inverter->SetOutputScalarTypeToUnsignedChar();
          inverter->SetShift(-255.0);
          inverter->SetScale(-1.0);
          inverter->Update();

          chartXY->GetBackgroundBrush()->SetTexture(inverter->GetOutput());
          chartXY->GetBackgroundBrush()->SetOpacity(255);
          chartXY->GetBackgroundBrush()->Modified();

        }else{
          chartXY->GetBackgroundBrush()->SetTexture(mModeFlipY);
          chartXY->GetBackgroundBrush()->SetOpacity(255);
          chartXY->GetBackgroundBrush()->Modified();
        }
      }else{
        if(ui->invertColorBox->isChecked()){
          //invert intensity
          vtkSmartPointer<vtkImageShiftScale> inverter =
        vtkSmartPointer<vtkImageShiftScale>::New();
          inverter->SetInput(mMode2);
          inverter->SetOutputScalarTypeToUnsignedChar();
          inverter->SetShift(-255.0);
          inverter->SetScale(-1.0);
          inverter->Update();

          chartXY->GetBackgroundBrush()->SetTexture(inverter->GetOutput());
          chartXY->GetBackgroundBrush()->SetOpacity(255);
          chartXY->GetBackgroundBrush()->Modified();
        }else{
          chartXY->GetBackgroundBrush()->SetTexture(mMode2);
          chartXY->GetBackgroundBrush()->SetOpacity(255);
          chartXY->GetBackgroundBrush()->Modified();
        }
      }      
    }else{
      if(ui->invertImageBox->isChecked()){  
        if(ui->invertColorBox->isChecked()){
          //invert intensity
          vtkSmartPointer<vtkImageShiftScale> inverter = 
        vtkSmartPointer<vtkImageShiftScale>::New();
          inverter->SetInput(mModeFlipY);
          inverter->SetOutputScalarTypeToUnsignedChar();
          inverter->SetShift(-255.0);
          inverter->SetScale(-1.0);
          inverter->Update();

          //render
          mModeMapper->SetInput(inverter->GetOutput());
        }else{
          //render
          mModeMapper->SetInput(mModeFlipY);
        }
      }else{
        if(ui->invertColorBox->isChecked()){
          //invert intensity
          vtkSmartPointer<vtkImageShiftScale> inverter = 
        vtkSmartPointer<vtkImageShiftScale>::New();
          inverter->SetInput(mMode2);
          inverter->SetOutputScalarTypeToUnsignedChar();
          inverter->SetShift(-255.0);
          inverter->SetScale(-1.0);
          inverter->Update();

          //render
          mModeMapper->SetInput(inverter->GetOutput());
        }else{
          //render
          mModeMapper->SetInput(mMode2);
        }
      }
      rendererMmode->ResetCamera();
    }
    mModewidget->GetRenderWindow()->Render();
  }else{
    //resize range of Slider
    ui->sequenceSlider->setRange(0,(imgVTK.size()*num)-1);
    ui->periodBeginSld->setRange(0,(imgVTK.size()*num)-1);
    ui->periodEndSld->setRange(0,(imgVTK.size()*num)-1);

    if(ui->graphicBox->isChecked()){ 
      chartXY->ClearPlots();
      fillPlot();
    
      mModewidget->GetRenderWindow()->Render();
    }
  }
      
}

void OpticalFlowWidget::invertIntensityBoxClicked(bool value){
  //invert intensity filter
  vtkSmartPointer<vtkImageShiftScale> inverter = 
    vtkSmartPointer<vtkImageShiftScale>::New();
    inverter->SetOutputScalarTypeToUnsignedChar();
  inverter->SetShift(-255.0);
  inverter->SetScale(-1.0);

  if(value){
    if(ui->invertImageBox->isChecked()){
      //invert intensity of invert image
      inverter->SetInput(mModeFlipY);
      inverter->Update();
    }else{
      //invert intensity
      inverter->SetInput(mMode2);
      inverter->Update();
    }
    if(ui->graphicBox->isChecked()){
      chartXY->GetBackgroundBrush()->SetTexture(inverter->GetOutput());
      chartXY->GetBackgroundBrush()->SetOpacity(255);
      chartXY->GetBackgroundBrush()->Modified();
    }else{
      //display inverter image
      mModeMapper->SetInput(inverter->GetOutput());
    }

  }else{
    if(ui->invertImageBox->isChecked()){
      if(ui->graphicBox->isChecked()){
        chartXY->GetBackgroundBrush()->SetTexture(mModeFlipY);
        chartXY->GetBackgroundBrush()->SetOpacity(255);
        chartXY->GetBackgroundBrush()->Modified();
      }else{
        //display inverter image
        mModeMapper->SetInput(mModeFlipY);
      }
    }else{
      if(ui->graphicBox->isChecked()){
        chartXY->GetBackgroundBrush()->SetTexture(mMode2);
        chartXY->GetBackgroundBrush()->SetOpacity(255);
        chartXY->GetBackgroundBrush()->Modified();
      }else{
        //display inverter image
        mModeMapper->SetInput(mMode2);
      }
    }
  }
  mModewidget->GetRenderWindow()->Render();
}

void OpticalFlowWidget::invertImageBoxClicked(bool value){  
  
  if(value){
    if(ui->invertColorBox->isChecked()){
      //invert intensity of inverted image
      vtkSmartPointer<vtkImageShiftScale> inverter = 
      vtkSmartPointer<vtkImageShiftScale>::New();
      inverter->SetInput(mModeFlipY);
      inverter->SetOutputScalarTypeToUnsignedChar();
      inverter->SetShift(-255.0);
      inverter->SetScale(-1.0);
      inverter->Update();

      if(ui->graphicBox->isChecked()){
        chartXY->GetBackgroundBrush()->SetTexture(inverter->GetOutput());
        chartXY->GetBackgroundBrush()->SetOpacity(255);
        chartXY->GetBackgroundBrush()->Modified();
      }else{
        //display inverter image
        mModeMapper->SetInput(inverter->GetOutput());
      }

    }else{
      if(ui->graphicBox->isChecked()){
        chartXY->GetBackgroundBrush()->SetTexture(mModeFlipY);
        chartXY->GetBackgroundBrush()->SetOpacity(255);
        chartXY->GetBackgroundBrush()->Modified();
      }else{
        //display inverter image
        mModeMapper->SetInput(mModeFlipY);
      }
    }
    
  }else{
    if(ui->invertColorBox->isChecked()){
      //invert image
      vtkSmartPointer<vtkImageShiftScale> inverter = 
      vtkSmartPointer<vtkImageShiftScale>::New();
      inverter->SetInput(mMode2);
      inverter->SetOutputScalarTypeToUnsignedChar();
      inverter->SetShift(-255.0);
      inverter->SetScale(-1.0);
      inverter->Update();

      if(ui->graphicBox->isChecked()){
        chartXY->GetBackgroundBrush()->SetTexture(inverter->GetOutput());
        chartXY->GetBackgroundBrush()->SetOpacity(255);
        chartXY->GetBackgroundBrush()->Modified();
      }else{
        //display inverter image
        mModeMapper->SetInput(inverter->GetOutput());
      }
    }else{
      if(ui->graphicBox->isChecked()){
        chartXY->GetBackgroundBrush()->SetTexture(mMode2);
        chartXY->GetBackgroundBrush()->SetOpacity(255);
        chartXY->GetBackgroundBrush()->Modified();
      }else{
        //display inverter image
        mModeMapper->SetInput(mMode2);
      }
    }    
  }
  mModewidget->GetRenderWindow()->Render();
}

void OpticalFlowWidget::changePeriodBeginSld(int p0){    
  //remove last references
  rendererMmode->RemoveActor(imageRefLineViewActorB);

  //save new position
  mModePeriod[0] = p0;

  //position of new reference
  imageRefLineViewB->SetPoint1(p0*mMode2->GetSpacing()[0],0,0);
  imageRefLineViewB->SetPoint2(p0*mMode2->GetSpacing()[0],
    mMode2->GetDimensions()[1]*mMode2->GetSpacing()[1],0);

  imageRefLineViewB->Update();    
  imageRefLineViewMapperB->Update();  

  rendererMmode->AddActor(imageRefLineViewActorB);
  mModewidget->GetRenderWindow()->Render();
}

void OpticalFlowWidget::changePeriodEndSld(int p1){    
  //remove last references
  rendererMmode->RemoveActor(imageRefLineViewActorE);

  //save new position
  mModePeriod[1] = p1;

  //position of new reference
  imageRefLineViewE->SetPoint1(p1*mMode2->GetSpacing()[0],0,0);
  imageRefLineViewE->SetPoint2(p1*mMode2->GetSpacing()[0],
    mMode2->GetDimensions()[1]*mMode2->GetSpacing()[1],0);

  imageRefLineViewE->Update();    
  imageRefLineViewMapperE->Update();  

  rendererMmode->AddActor(imageRefLineViewActorE);
  mModewidget->GetRenderWindow()->Render();
}

void OpticalFlowWidget::segmentBtnClicked(bool value){
      
  if(value){
    //remove vectors and scalar bar     
    viewerImage->GetRenderer()->RemoveViewProp(vectorActor);
    viewerImage->GetRenderer()->RemoveViewProp(scalarBarActor);
    viewerImage->GetRenderer()->RemoveViewProp(segmentActor);

    //disable sequence slider
    ui->sequenceSlider->setDisabled(true);
    
    //display first image of secuence
    segImgInd = 0;
    viewerImage->SetInput(imgVTK[segImgInd]);
    viewerImage->Render();  

    //vectors
    segScalarBarActors.resize(imgVTK.size());
    segVectorActors.resize(imgVTK.size());
    segSplineActors.resize(imgVTK.size());
    segPoints.resize(imgVTK.size());
    
    //initialize segmentation interactor style
    segmentStyle = vtkSmartPointer< vtkTracerInteractorStyle
		<OpticalFlowWidget> >::New();
    segmentStyle->setCallerWidget(this);
    viewerImage->GetRenderWindow()->GetInteractor()->
		SetInteractorStyle(segmentStyle);
    segmentStyle->setAutoCloseOff();
    //add m-mode interactor style to selected view 
    segmentStyle->initTracer(imageActor);    
    
    ui->segmentBtn->setDisabled(true);
    
  }else{
    ui->segmentBtn->setEnabled(true);
    ui->segmentBtn->setChecked(false);
    //removing m-mode interactor style from selected view 
    segmentStyle->clearTracer();

    viewerImage->GetRenderer()->AddActor(imageActor);
    viewerImage->GetRenderWindow()->GetInteractor()->
		SetInteractorStyle(imageStyleImage);

    //Display image
    viewerImage->SetInput(imgVTK[ui->sequenceSlider->value()-vtkMath::
	  Floor(ui->sequenceSlider->value()/imgVTK.size())*(imgVTK.size())]);
    
    ui->sequenceSlider->setEnabled(true);

    //Graphic
    chartXY->ClearPlots();
    fillPlot();
    ui->graphicBox->setEnabled(true);

    //vectors
    ui->onSegmentRBtn->setEnabled(true);
    ui->onSegmentRBtn->setChecked(true);

    rendererMmode->ResetCamera();
    mModewidget->GetRenderWindow()->Render();

    //viewOFBoxClicked(ui->viewOFBox->isChecked()); 
    changeDisplayedSeq(ui->sequenceSlider->value());
  }
}

void OpticalFlowWidget::setSegmentedPath(vtkSmartPointer<vtkPolyData> anotation){
  ui->imageOFwidget->setDisabled(true);

  segmentData = anotation;
  
  //dialog window
  OFsegWidget *widget = new OFsegWidget;
  widget->setWindowTitle("Segmentation Done");
  widget->setWindowFlags(Qt::CustomizeWindowHint | 
    Qt::WindowTitleHint);//hide close and maximixe bottons
  widget->setOpticalFlowWidget(this);
  widget->show();  
  
}

void OpticalFlowWidget::segmentedPoints(bool value){

  if( value){
    //number of points in all splines
    int nPnt = 120;

    //spline 
    vtkSmartPointer<vtkParametricSpline> spline = 
    vtkSmartPointer<vtkParametricSpline>::New();

    vtkSmartPointer<vtkKochanekSpline> splineX = 
    vtkSmartPointer<vtkKochanekSpline>::New();
    splineX->SetDefaultTension(0.7);

    vtkSmartPointer<vtkKochanekSpline> splineY = 
    vtkSmartPointer<vtkKochanekSpline>::New();
    splineY->SetDefaultTension(0.7);

    spline->SetXSpline( splineX );
    spline->SetYSpline( splineY );
    spline->ClosedOff();
    spline->SetPoints(segmentData->GetPoints());
    
    vtkSmartPointer<vtkParametricFunctionSource> functionSource = 
    vtkSmartPointer<vtkParametricFunctionSource>::New();
    functionSource->SetParametricFunction(spline);
    functionSource->SetUResolution(nPnt-1);//spline make nPnt+1 points
    functionSource->SetVResolution(nPnt-1);//spline make nPnt+1 points
    functionSource->Update();

    double vect[3];

    double* spacing = imgVTK[0]->GetSpacing();

    segPoints[segImgInd].resize(nPnt);

    //transform coordinates to integer 
    for(vtkIdType i = 0; i < nPnt; i++)
    {
      segPoints[segImgInd][i].resize(2);
      segPoints[segImgInd][i][0] = vtkMath::Floor((functionSource->
        GetOutput()->GetPoints()->GetPoint(i)[0]/spacing[0]) + 0.5);
      segPoints[segImgInd][i][1] = vtkMath::Floor((functionSource->
        GetOutput()->GetPoints()->GetPoint(i)[1]/spacing[1]) + 0.5);
    }

    //segmented line to actor for draw
    vtkSmartPointer<vtkPolyDataMapper> segMapper = 
    vtkSmartPointer<vtkPolyDataMapper>::New();
      segMapper->SetInputConnection(functionSource->GetOutputPort());

    segSplineActors[segImgInd] = vtkSmartPointer<vtkActor>::New();
    segSplineActors[segImgInd]->SetMapper(segMapper);
    segSplineActors[segImgInd]->GetProperty()->SetColor(0,1,0);
    segSplineActors[segImgInd]->GetProperty()->SetLineWidth(1.0);

    cout<<" Contour "<<segImgInd<<" saved "<<std::endl;      

    //get next image 
    if(segImgInd+1< imgVTK.size()){
      segImgInd = segImgInd+1;
      //removing m-mode interactor style from selected view 
      segmentStyle->clearTracer();
      viewerImage->SetInput(imgVTK[segImgInd]);
      //add m-mode interactor style to selected view 
      segmentStyle->initTracer(imageActor);  
    }else{
      //save seg points
      oldSegPoints = segPoints;

      //save line to actor for draw
      oldSegSplineActors = segSplineActors;

      //create actors
      createSegVectors();

      //segment botton = false
      ui->segmentBtn->setChecked(false);
      segmentBtnClicked(false);
    }
  }else{
    //removing interactor style from selected view 
    segmentStyle->clearTracer();
    viewerImage->SetInput(imgVTK[segImgInd]);
    //add interactor style to selected view 
    segmentStyle->initTracer(imageActor);  
  }

  ui->imageOFwidget->setEnabled(true);
}

void OpticalFlowWidget::clearSegmentation(){
  segImgInd = 0;

  cout<<"  All recent contours have been deleted "<<std::endl;  
  //old values
  segPoints = oldSegPoints;
  segSplineActors = oldSegSplineActors;

  //removing m-mode interactor style from selected view 
  segmentStyle->clearTracer();

  viewerImage->GetRenderer()->AddActor(imageActor);
  viewerImage->GetRenderWindow()->GetInteractor()->
	  SetInteractorStyle(imageStyleImage);

  //Display image
  viewerImage->SetInput(imgVTK[ui->sequenceSlider->value() - 
    vtkMath::Floor(ui->sequenceSlider->value()/imgVTK.size())*
    (imgVTK.size())]);

  changeDisplayedSeq(ui->sequenceSlider->value());

  ui->segmentBtn->setChecked(false);

  ui->sequenceSlider->setEnabled(true);
  ui->segmentBtn->setEnabled(true);
  ui->imageOFwidget->setEnabled(true);
}

void OpticalFlowWidget::graphicBoxClicked(bool value){  
  if (value){

    chartXY->ClearPlots();
    fillPlot();  
    
    if(mModeFlag){      
      ui->periodBeginSld->setDisabled(true);
      ui->periodEndSld->setDisabled(true);

      rendererMmode->RemoveActor(imageRefLineViewActor);
      rendererMmode->RemoveActor(imageRefLineViewActorB);
      rendererMmode->RemoveActor(imageRefLineViewActorE);

      if(ui->invertImageBox->isChecked()){  
        if(ui->invertColorBox->isChecked()){
          //invert intensity
          vtkSmartPointer<vtkImageShiftScale> inverter = 
        vtkSmartPointer<vtkImageShiftScale>::New();
          inverter->SetInput(mModeFlipY);
          inverter->SetOutputScalarTypeToUnsignedChar();
          inverter->SetShift(-255.0);
          inverter->SetScale(-1.0);
          inverter->Update();

          chartXY->GetBackgroundBrush()->SetTexture
			  (inverter->GetOutput());
          chartXY->GetBackgroundBrush()->SetOpacity(255);
          chartXY->GetBackgroundBrush()->Modified();

        }else{
          chartXY->GetBackgroundBrush()->SetTexture(mModeFlipY);
          chartXY->GetBackgroundBrush()->SetOpacity(255);
          chartXY->GetBackgroundBrush()->Modified();
        }
      }else{
        if(ui->invertColorBox->isChecked()){
          //invert intensity
          vtkSmartPointer<vtkImageShiftScale> inverter = 
        vtkSmartPointer<vtkImageShiftScale>::New();
          inverter->SetInput(mMode2);
          inverter->SetOutputScalarTypeToUnsignedChar();
          inverter->SetShift(-255.0);
          inverter->SetScale(-1.0);
          inverter->Update();

          chartXY->GetBackgroundBrush()->SetTexture
			  (inverter->GetOutput());
          chartXY->GetBackgroundBrush()->SetOpacity(255);
          chartXY->GetBackgroundBrush()->Modified();
        }else{
          chartXY->GetBackgroundBrush()->SetTexture(mMode2);
          chartXY->GetBackgroundBrush()->SetOpacity(255);
          chartXY->GetBackgroundBrush()->Modified();
        }
      }

      rendererMmode->RemoveViewProp(mModeSlice);
      //plots and texture match but origin is 0,0
      chartXY->SetLayoutStrategy(2);
      /*pos x = bound bottom = 83, 
        pos y = bound left = 33 , 
        widith =  RenderWindow x - (bound left + bound rigth) , 
        higth =  RenderWindow y - (bound bottom + bound top)*/
      chartXY->SetSize(vtkRectf(83,33,mModewidget->GetRenderWindow()->
      GetSize()[0]-90,mModewidget->GetRenderWindow()->GetSize()[1]-40));
  
      chartXY->Update();

    }

    rendererMmode->AddViewProp(contextMmodeActor);
    
  }else{
    rendererMmode->RemoveViewProp(contextMmodeActor);
    
    if(mModeFlag){        
      ui->periodBeginSld->setEnabled(true);
      ui->periodEndSld->setEnabled(true);

      //position of new reference
      imageRefLineView->SetPoint1(ui->sequenceSlider->value()*
      mMode2->GetSpacing()[0],0,0);
      imageRefLineView->SetPoint2
		  (ui->sequenceSlider->value()*mMode2->GetSpacing()[0],
      mMode2->GetDimensions()[1]*mMode2->GetSpacing()[1],0);      

      rendererMmode->AddActor(imageRefLineViewActor);
      rendererMmode->AddActor(imageRefLineViewActorB);
      rendererMmode->AddActor(imageRefLineViewActorE);

      if(ui->invertImageBox->isChecked()){  
        if(ui->invertColorBox->isChecked()){
          //invert intensity
          vtkSmartPointer<vtkImageShiftScale> inverter = 
        vtkSmartPointer<vtkImageShiftScale>::New();
          inverter->SetInput(mModeFlipY);
          inverter->SetOutputScalarTypeToUnsignedChar();
          inverter->SetShift(-255.0);
          inverter->SetScale(-1.0);
          inverter->Update();

          //display
          mModeMapper->SetInput(inverter->GetOutput());
        }else{
          //display
          mModeMapper->SetInput(mModeFlipY);
        }
      }else{
        if(ui->invertColorBox->isChecked()){
          //invert intensity
          vtkSmartPointer<vtkImageShiftScale> inverter = 
        vtkSmartPointer<vtkImageShiftScale>::New();
          inverter->SetInput(mMode2);
          inverter->SetOutputScalarTypeToUnsignedChar();
          inverter->SetShift(-255.0);
          inverter->SetScale(-1.0);
          inverter->Update();

          //display
          mModeMapper->SetInput(inverter->GetOutput());
        }else{
          //display
          mModeMapper->SetInput(mMode2);
        }
      }

      rendererMmode->AddViewProp(mModeSlice);
      mModewidget->GetRenderWindow()->Render();
      rendererMmode->ResetCamera();
    }    
  }

  mModewidget->GetRenderWindow()->Render();
  
}

void OpticalFlowWidget:: scaleVectBox(double value) {
  scale = value;

  //remove vectors and scalar bar     
  viewerImage->GetRenderer()->RemoveViewProp(vectorActor);
  viewerImage->GetRenderer()->RemoveViewProp(scalarBarActor);
  
  //generate actors 
  createAllVectors();

  if(ui->onSegmentRBtn->isEnabled()){
    createSegVectors();
  }
  
  if(ui->viewOFBox->isChecked()){
    //actualize  actor
    if(ui->onSegmentRBtn->isChecked()){
      scalarBarActor = segScalarBarActors[frame];
      vectorActor = segVectorActors[frame];
    }else{
      scalarBarActor = allScalarBarActors[frame];
      vectorActor = allVectorActors[frame];
    }
    //Display vectors and scalar bar
    viewerImage->GetRenderer()->AddViewProp(vectorActor);
    viewerImage->GetRenderer()->AddViewProp(scalarBarActor);
  }

  viewerImage->Render();  
}

void OpticalFlowWidget::createAllVectors(){
  
  int* dims = imgVTK[0]->GetDimensions();
  double* spacing = imgVTK[0]->GetSpacing();

  allScalarBarActors.resize(imgVTK.size());
  allVectorActors.resize(imgVTK.size());

  for(int imageN=0; imageN<imgVTK.size(); imageN++){
    // Create Position
    vtkSmartPointer<vtkPoints> points = 
		vtkSmartPointer<vtkPoints>::New();
    
    // Create Vector
    vtkSmartPointer<vtkDoubleArray> vects = 
    vtkSmartPointer<vtkDoubleArray>::New();
    vects->SetNumberOfComponents(3);

    double vect[3];
    //set coordinates of the image 
    itk::Image<double, 2>::IndexType index;
    
    //every 4 pixels draw one vector
    for(int i=0; i<dims[0]; i=i+4){
      for(int j=0; j<dims[1]; j=j+4){
    
        index[0]=i; index[1]= j;
        //values: vect = {valueU[0],valueV[0],0};//{valX,valY,0}
        //escaling vector with spacing (value*spacing)
        vect[0] = scale*duITK[imageN]->GetPixel(index)*spacing[0];
        vect[1] = scale*dvITK[imageN]->GetPixel(index)*spacing[1];
        
        vect[2] = 0.0;

        /*
		position = (posX+(valX/2),posY+(valY/2),0); 
        pos+(val/2) -> begining of arrow in the center of pixel
        points->InsertNextPoint(
		  (i*spacing[0])+ (duITK[imageN]->GetPixel(index)*spacing[0]/2),
		  (j*spacing[1])+(dvITK[imageN]->GetPixel(index)*spacing[1]/2),
		  0.0);
		*/

        points->InsertNextPoint((i*spacing[0])+(vect[0]/2),
			(j*spacing[1])+(vect[1]/2),0.0);
        
        vects->InsertNextTupleValue(vect);
      }
    }
        
    // Combine into a polydata  
    vtkSmartPointer<vtkPolyData> polydata = 
		vtkSmartPointer<vtkPolyData>::New();
    polydata->SetPoints(points);
    polydata->GetPointData()->SetVectors(vects);

    //range = max magnitude
    double range[2];
    polydata->GetPointData()->GetVectors()->GetRange(range,-1);
    
    // Setup the arrows
    vtkGlyphSource2D* arrow = vtkGlyphSource2D::New();
    arrow->SetGlyphTypeToArrow();
    arrow->FilledOff();
    
    vtkSmartPointer<vtkGlyph2D> glyphFilter = 
		vtkSmartPointer<vtkGlyph2D>::New();
    glyphFilter->SetSourceConnection(arrow->GetOutputPort());
    glyphFilter->OrientOn();
    glyphFilter->SetVectorModeToUseVector();
    glyphFilter->SetScaleModeToScaleByVector();//use magnitude of vector
    glyphFilter->SetColorModeToColorByVector();
    glyphFilter->SetInput(polydata);
    glyphFilter->Update();

    //Create a lookup table to map data to colors
    vtkSmartPointer<vtkLookupTable> lut = 
		vtkSmartPointer<vtkLookupTable>::New();
    lut->SetTableRange (0.5, 1);
    lut->SetHueRange (0.5, 1);
    lut->SetSaturationRange (1, 1);
    lut->SetValueRange (1, 1);
    lut->Build();

    //scalar bar for lookup table
     vtkSmartPointer<vtkScalarBarActor> ofScalarBarActor = 
     vtkSmartPointer<vtkScalarBarActor>::New();
     ofScalarBarActor->SetTitle("Magnitude");
     ofScalarBarActor->SetNumberOfLabels(10);
    
    // Create actors 
    vtkSmartPointer<vtkPolyDataMapper> vectorMapper = 
    vtkSmartPointer<vtkPolyDataMapper>::New();
    //for ColorModeToColorByVector with magnitude of 
	  //vector (min value, max value)
    vectorMapper->SetScalarRange(range);
	vectorMapper->SetLookupTable( lut );
    ofScalarBarActor->SetLookupTable(vectorMapper->GetLookupTable());
    vectorMapper->ScalarVisibilityOn();
    vectorMapper->SetColorModeToMapScalars();
    vectorMapper->SetInputConnection(glyphFilter->GetOutputPort());

    vtkSmartPointer<vtkActor> ofVectorActor = 
    vtkSmartPointer<vtkActor>::New();
    ofVectorActor->SetMapper(vectorMapper);

    allScalarBarActors[imageN] = vtkSmartPointer<vtkScalarBarActor>::New();
    allScalarBarActors[imageN] = ofScalarBarActor;

    allVectorActors[imageN] = vtkSmartPointer<vtkActor>::New();
    allVectorActors[imageN] = ofVectorActor;
  }  
}

void OpticalFlowWidget::createSegVectors(){
  int* dims = imgVTK[0]->GetDimensions();
  double* spacing = imgVTK[0]->GetSpacing();


  for(int imageN=0; imageN<imgVTK.size(); imageN++){
    // Create Position
    vtkSmartPointer<vtkPoints> points = vtkSmartPointer<vtkPoints>::New();
    
    // Create Vector
    vtkSmartPointer<vtkDoubleArray> vects = 
		vtkSmartPointer<vtkDoubleArray>::New();
    vects->SetNumberOfComponents(3);

    double vect[3];

    //set coordinates of the image 
    itk::Image<double, 2>::IndexType index;
    for(vtkIdType i = 0; i < segPoints[imageN].size();i++){
      //dont repeat same point
      if( (i==0)||((i!=0)&&
		  ( (segPoints[imageN][i][0]!=segPoints[imageN][i-1][0])||
          (segPoints[imageN][i][1]!=segPoints[imageN][i-1][1]) )) ){
      
	    index[0] = segPoints[imageN][i][0];
        index[1] = segPoints[imageN][i][1];
        
        //values: vect = {valueU[0],valueV[0],0};//{valX,valY,0}
        //escaling vector with spacing (value*spacing)
        vect[0] = scale*duITK[imageN]->GetPixel(index)*spacing[0];
        vect[1] = scale*dvITK[imageN]->GetPixel(index)*spacing[1];
        
        vect[2] = 0.0;

        /*position (posX+(valX/2),posY+(valY/2),0); 
         pos+(val/2) -> begining of arrow in the center of pixel*/
        points->InsertNextPoint((index[0]*spacing[0])+(vect[0]/2),
          (index[1]*spacing[1])+(vect[1]/2),0.0);
        
        vects->InsertNextTupleValue(vect);
      }
    }
        
    // Combine into a polydata  
    vtkSmartPointer<vtkPolyData> polydata = vtkSmartPointer<vtkPolyData>::New();
    polydata->SetPoints(points);
    polydata->GetPointData()->SetVectors(vects);

    //range = max magnitude
    double range[2];
    polydata->GetPointData()->GetVectors()->GetRange(range,-1);
    
    // Setup the arrows
    vtkGlyphSource2D* arrow = vtkGlyphSource2D::New();
    arrow->SetGlyphTypeToArrow();
    arrow->FilledOff();
    
    vtkSmartPointer<vtkGlyph2D> glyphFilter = vtkSmartPointer<vtkGlyph2D>::New();
    glyphFilter->SetSourceConnection(arrow->GetOutputPort());
    glyphFilter->OrientOn();
    glyphFilter->SetVectorModeToUseVector();
    glyphFilter->SetScaleModeToScaleByVector();//use magnitude of vector
    glyphFilter->SetColorModeToColorByVector();
    glyphFilter->SetInput(polydata);
    glyphFilter->Update();

    //Create a lookup table to map data to colors
    vtkSmartPointer<vtkLookupTable> lut = vtkSmartPointer<vtkLookupTable>::New();
    lut->SetTableRange (0.5, 1);
    lut->SetHueRange (0.5, 1);
    lut->SetSaturationRange (1, 1);
    lut->SetValueRange (1, 1);
    lut->Build();

    //scalar bar for lookup table
     vtkSmartPointer<vtkScalarBarActor> ofScalarBarActor = 
     vtkSmartPointer<vtkScalarBarActor>::New();
     ofScalarBarActor->SetTitle("Magnitude");
     ofScalarBarActor->SetNumberOfLabels(10);
    
    // Create actors 
    vtkSmartPointer<vtkPolyDataMapper> vectorMapper = 
    vtkSmartPointer<vtkPolyDataMapper>::New();
    vectorMapper->SetScalarRange(range);
    vectorMapper->SetLookupTable( lut );
    ofScalarBarActor->SetLookupTable(vectorMapper->GetLookupTable());
    vectorMapper->ScalarVisibilityOn();
    vectorMapper->SetColorModeToMapScalars();
    vectorMapper->SetInputConnection(glyphFilter->GetOutputPort());

    vtkSmartPointer<vtkActor> ofVectorActor = vtkSmartPointer<vtkActor>::New();
    ofVectorActor->SetMapper(vectorMapper);

    segScalarBarActors[imageN] = vtkSmartPointer<vtkScalarBarActor>::New();
    segScalarBarActors[imageN] = ofScalarBarActor;

    segVectorActors[imageN] = vtkSmartPointer<vtkActor>::New();
    segVectorActors[imageN] = ofVectorActor;
  }  
}

void OpticalFlowWidget::fillPlot(){
  
  vtkSmartPointer<vtkTable> table = vtkSmartPointer<vtkTable>::New();
        
  double magnitude;
  int numPoints = imgVTK.size();  

  //columns in table of graphic
  vtkSmartPointer<vtkDoubleArray> arrX = vtkSmartPointer<vtkDoubleArray>::New();
  arrX->SetName("Image");
  table->AddColumn(arrX);

  char name[15];

  //number of columns for poinst ins segment vectors
    // all segmentations habe same number of points
  for(int i=1; i<=segPoints[0].size()+1; i++){
    vtkSmartPointer<vtkDoubleArray> arrV = 
		vtkSmartPointer<vtkDoubleArray>::New();
    sprintf(name, "Vector_%i",i);
    arrV->SetName(name);//is necesary for create column
    table->AddColumn(arrV);
  }

  table->SetNumberOfRows((numPoints*mModeCicles)+1);

  //axix values
  for (int k = 0; k < (numPoints*mModeCicles)+1 ; k++){
    table->SetValue(k, 0, k);
  }
  
  double* spacing = imgVTK[0]->GetSpacing();

  itk::Image<double, 2>::IndexType index;

  double prom;
  for (int c = 0; c < mModeCicles; c++){    
    for (int k = 0; k < numPoints; ++k){
      prom = 0;
      for(int i=0; i < segPoints[0].size(); i++){
        index[0] = segPoints[k][i][0];
        index[1] = segPoints[k][i][1];

        /*magnitude = sqrt(( pow(duITK[k]->GetPixel(index),2) )+
         ( pow(dvITK[k]->GetPixel(index),2) ));*/
        magnitude = sqrt(( pow(duITK[k]->GetPixel(index)*spacing[0],2) )+
                         ( pow(dvITK[k]->GetPixel(index)*spacing[1],2) ));
                     //(row             ,col  ,value)
        table->SetValue(k+(c*numPoints) , i+1, magnitude );  

        prom = prom + magnitude;
      }
      table->SetValue( k+(c*numPoints), segPoints[0].size()+1, 
		  prom/segPoints[0].size() );
    }
  }

  //end of graphic is the begining of mMode2s texture -> last value = first value
  prom = 0;
  for(int i=0; i < segPoints[0].size(); i++){
    index[0] = segPoints[0][i][0];
    index[1] = segPoints[0][i][1];

    magnitude = sqrt(( pow(duITK[0]->GetPixel(index)*spacing[0],2) )+
    ( pow(dvITK[0]->GetPixel(index)*spacing[1],2) ));
    
    table->SetValue( numPoints*mModeCicles, i+1, magnitude );  

    prom = prom + magnitude;
  }
  table->SetValue( numPoints*mModeCicles, segPoints[0].size()+1, 
	  prom/segPoints[0].size() );

  for(int i=1; i<=segPoints[0].size(); i++){
    plotLine = chartXY->AddPlot(vtkChart::LINE);
    plotLine->SetInput(table,0,i);
    plotLine->SetColor(0, 255, 0, 255);//(r,g,b,a)
    plotLine->SetWidth(1.0);

  }

  //prom
  plotLine = chartXY->AddPlot(vtkChart::LINE);
  plotLine->SetInput(table,0,segPoints[0].size()+1);
  plotLine->SetColor(0, 0,255, 255);//(r,g,b,a)
  plotLine->SetWidth(2.0);

  plotLine->GetXAxis()->SetNotation(0);//standar
  plotLine->GetYAxis()->SetNotation(1);//cientific

  plotLine->SetOpacity(0.5);

  plotLine->Update();
  
  chartXY->SetForceAxesToBounds(true);

  if(mModeFlag){
    if(ui->invertImageBox->isChecked()){  
      if(ui->invertColorBox->isChecked()){
        //invert intensity
        vtkSmartPointer<vtkImageShiftScale> inverter = 
          vtkSmartPointer<vtkImageShiftScale>::New();
        inverter->SetInput(mModeFlipY);
        inverter->SetOutputScalarTypeToUnsignedChar();
        inverter->SetShift(-255.0);
        inverter->SetScale(-1.0);
        inverter->Update();

        chartXY->GetBackgroundBrush()->SetTexture(inverter->GetOutput());
        chartXY->GetBackgroundBrush()->SetOpacity(255);
        chartXY->GetBackgroundBrush()->Modified();

      }else{
        chartXY->GetBackgroundBrush()->SetTexture(mModeFlipY);
        chartXY->GetBackgroundBrush()->SetOpacity(255);
        chartXY->GetBackgroundBrush()->Modified();
      }
    }else{
      if(ui->invertColorBox->isChecked()){
        //invert intensity
        vtkSmartPointer<vtkImageShiftScale> inverter = 
          vtkSmartPointer<vtkImageShiftScale>::New();
        inverter->SetInput(mMode2);
        inverter->SetOutputScalarTypeToUnsignedChar();
        inverter->SetShift(-255.0);
        inverter->SetScale(-1.0);
        inverter->Update();

        chartXY->GetBackgroundBrush()->SetTexture(inverter->GetOutput());
        chartXY->GetBackgroundBrush()->SetOpacity(255);
        chartXY->GetBackgroundBrush()->Modified();
      }else{
        chartXY->GetBackgroundBrush()->SetTexture(mMode2);
        chartXY->GetBackgroundBrush()->SetOpacity(255);
        chartXY->GetBackgroundBrush()->Modified();
      }
    }
  }
  
  chartXY->Update();
}
