/*
 * File:   Optical Flow
 * Author: Fabiola Pacheco
 * 
 * Created on 08/18/2017
 * Last modification on 09/07/2017
 */
#ifndef OPTICALFLOWWIDGET_H
#define OPTICALFLOWWIDGET_H

#include <QWidget>
#include <QVTKWidget.h>
#include "mainwindow.h"

#include <vtkScalarBarActor.h>
#include <vtkDoubleArray.h>
#include <vtkPoints.h>
#include <vtkPointData.h>
#include <vtkGlyph2D.h>
#include <vtkLookupTable.h>
#include <vtkLineSource.h>
#include <vtkFieldData.h>
#include <vtkImageSliceMapper.h>
#include <vtkPolyDataMapper.h>
#include <vtkImageSlice.h>
#include <vtkChartXY.h>
#include <vtkPlot.h>
#include "vtkContextActor.h"


namespace Ui {
class OpticalFlowWidget;
}

class OpticalFlowWidget : public QWidget
{
    Q_OBJECT

public:
    explicit OpticalFlowWidget(QWidget *parent = 0);
    ~OpticalFlowWidget();

  /**
  Set the secuence of images and coordinates of vectos of Optical Flow to widget
    (vector of secuence of images,vector of coordinates du,vector of coordinates dv)
  **/
  void setOpticalFlowData(std::vector< vtkSmartPointer<vtkImageData> >,
                          std::vector< itk::Image<double,2>::Pointer >,
                          std::vector< itk::Image<double,2>::Pointer >); 
  
  /**
  Set the M-mode of secuence and parameters
    (image M-Mode,begin and end of selected period,times repited M-Mode for calaculated period)
  **/
  void setMmodeData(vtkSmartPointer<vtkImageData>,int[2],int);
  
  /**
  Take polydata of the segmentation interactor style
  **/
  void setSegmentedPath(vtkSmartPointer<vtkPolyData>);

  /**
  Get Image Points of segmentation
  **/
  void segmentedPoints(bool);

  /**
  Clean segmentation with outsave
  **/
  void clearSegmentation();

private:
    Ui::OpticalFlowWidget *ui;

  /**
  Widget that display the Optical Flow
  **/
  QVTKWidget *imageOFwidget;

  /**
  Widget that display the M-Mode
  **/
  QVTKWidget *mModewidget;

  /**
  save secuence of VTK images for visualization of Optical Flow
  **/
  std::vector< vtkSmartPointer<vtkImageData> > imgVTK;  
  
  /**
  save coordinates vectors of optical flow in ITK 2d images
  **/
  std::vector< itk::Image<double,2>::Pointer > duITK;
  std::vector< itk::Image<double,2>::Pointer > dvITK;
  
  /**
  Image Viewer that render the image
  **/
  vtkSmartPointer<vtkImageViewer2> viewerImage;

  /**
  ImageActor of the image
  **/
  vtkSmartPointer<vtkImageActor> imageActor;

  /**
  Interactor style of the image viewer, it has the basic interaction with a 2D image
  **/
  vtkSmartPointer<vtkInteractorStyleImage> imageStyleImage;

  /**
  Scalar bar for lookup table for graphic vectors of optical flow
  **/
  vtkSmartPointer<vtkScalarBarActor> scalarBarActor;
    
  /**
  Actor for graphic vectors of optical flow
  **/
  vtkSmartPointer<vtkActor> vectorActor;

  /**
  Actor for segmentation on optical flow
  **/
  vtkSmartPointer<vtkActor> segmentActor;

  /**
  Scalar bar for lookup table for all vectors of optical flow
  **/
  std::vector< vtkSmartPointer<vtkScalarBarActor> > allScalarBarActors;
    
  /**
  Actor for all vectors of optical flow
  **/
  std::vector< vtkSmartPointer<vtkActor> > allVectorActors;

  /**
  points in segmentation
  **/
  std::vector< std::vector< std::vector< unsigned int > > > oldSegPoints;
  std::vector< std::vector< std::vector< unsigned int > > > segPoints;

  /**
  Scalar bar for lookup table for the vectors of optical flow in segmentation
  **/
  std::vector< vtkSmartPointer<vtkScalarBarActor> > segScalarBarActors;
    
  /**
  Actors of the vectors of optical flow in segmentation
  **/
  std::vector< vtkSmartPointer<vtkActor> > segVectorActors;

  /**
  Actors of segmentation
  **/
  std::vector<vtkSmartPointer<vtkActor> > oldSegSplineActors;
  std::vector<vtkSmartPointer<vtkActor> > segSplineActors;

  /**
  Create all vectors for display
  **/
  void createAllVectors();

  /**
  Create segmented vectors for display
  **/
  void createSegVectors();

  /**
  save M-Mode
  **/
  vtkSmartPointer<vtkImageData> mMode;

  /**
  M-Mode n times
  **/
  vtkSmartPointer<vtkImageData> mMode2;

  /**
  flip repeat M-Mode image
  **/
  vtkSmartPointer<vtkImageData> mModeFlipY;

  /**
  times of repeated M-Mode
  **/
  int mModeCicles;

  /**
  begin and enf of a period in M-Mode
  **/
  int mModePeriod[2];

  /**
  flag M-Mode, if M-Mode exist then flag == true 
  **/
  bool mModeFlag;

  /**
  Image Renderer of the M-Mode widget
  **/
  vtkSmartPointer<vtkRenderer> rendererMmode;

  /**
  Image Maper of the M-Mode
  **/
  vtkSmartPointer<vtkImageSliceMapper> mModeMapper;  
  
  /**
  Image Slice of the M-Mode
  **/
  vtkSmartPointer<vtkImageSlice> mModeSlice;

  /**
  Interactor style of the image viewer of M-Mode
  **/
  vtkSmartPointer<vtkInteractorStyle> imageStyleImageMmode;

  /**
  M-Modes reference in view
  **/
  vtkSmartPointer<vtkLineSource> imageRefLineView;

  /**
  M-Modes reference in view actor
  **/
  vtkSmartPointer<vtkActor> imageRefLineViewActor;

  /**
  M-Modes reference in view mapper
  **/
  vtkSmartPointer<vtkPolyDataMapper> imageRefLineViewMapper;

  /**
  Begining and Ending of Periods  references in view
  **/
  vtkSmartPointer<vtkLineSource> imageRefLineViewB;
  vtkSmartPointer<vtkLineSource> imageRefLineViewE;

  /**
  Begining and Ending of Periods  references in view actor
  **/
  vtkSmartPointer<vtkActor> imageRefLineViewActorB;
  vtkSmartPointer<vtkActor> imageRefLineViewActorE;

  /**
  Begining and Ending of Periods  references in view mapper
  **/
  vtkSmartPointer<vtkPolyDataMapper> imageRefLineViewMapperB;
  vtkSmartPointer<vtkPolyDataMapper> imageRefLineViewMapperE;

  /**
  Segmented interactor style
  **/
  vtkSmartPointer< vtkTracerInteractorStyle<OpticalFlowWidget> > segmentStyle;

  /**
  chart XY
  **/
  vtkSmartPointer< vtkChartXY > chartXY;

  /**
  plot for graphics
  **/
  vtkSmartPointer<vtkPlot> plotLine;

  /**
  Grapics actor
  **/
  vtkSmartPointer<vtkContextActor> contextMmodeActor;

  /**
  Fill table of points for graphicas 
  **/
  void fillPlot();

  /**
  Index of actual image in segmentation
  **/
  unsigned int segImgInd;

  /**
  Polydata of segmentation
  **/
  vtkSmartPointer<vtkPolyData> segmentData;

  /**
  Number of actual image
  **/
  unsigned int frame;

  /**
    Scale of Vectors
    **/
  double scale;

private slots:
  /**
    Set all vectors in render
    **/
    void viewOFBoxClicked(bool);

  /**
    Set segmented vectors in render
    **/
    void vectSegRBtnClicked(bool);

  /**
  Change the displayed sequence
  **/
  void changeDisplayedSeq(int);

  /**
    Set how many times is M-Mode repeated
    **/
  void setCicles(int);

  /**
    Set inversion of intensity of M-Mode
    **/
    void invertIntensityBoxClicked(bool);

  /**
    Set inversion of M-Mode
    **/
    void invertImageBoxClicked(bool);

  /**
    Set period in M-Mode
    **/
  void changePeriodBeginSld(int);  
  void changePeriodEndSld(int);  

  /**
    Initialize tracer for manual segmentation
    **/
  void segmentBtnClicked(bool);

  /**
    Set Grophics of Vectors in M-Mode
    **/
    void graphicBoxClicked(bool);

  /**
    Scale Vectors
    **/
    void scaleVectBox(double);

};

#endif // OPTICALFLOWWIDGET_H
